# -*- coding: UTF-8 -*-
from __future__ import division
import execjs
from json import *
import urllib
import urllib2
import cookielib
import re
import sys
import os

reload(sys)
sys.setdefaultencoding('utf-8')

zh_pattern = re.compile(u'[\u4e00-\u9fa5]+')


def contain_zh(word):
    word = word.decode()
    global zh_pattern
    match = zh_pattern.search(word)
    return match


user_agent = "Mozilla/5.0 (Windows NT 6.1; Win64; x64) " \
             "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36"


class NetOpener(object):
    def __init__(self, host, cookie_fname):
        # 配置cookie信息，cookie文件路径根据自己需要配置
        self.cookie_dir = "".join(["cookies/", cookie_fname])

        if not os.path.isdir("cookies/"):
            os.makedirs("cookies/")

        self.headers = {
            "User-Agent": user_agent,
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Content-Type": "application/json",
            "Origin": host,
            "Referer": host
        }
        self.host = host
        self.cookie = None
        self.opener = None

    # 初始化cookie
    def init_cookie(self):
        cookie = cookielib.MozillaCookieJar(self.cookie_dir)
        handler = urllib2.HTTPCookieProcessor(cookie)
        opener = urllib2.build_opener(handler)
        opener.addheaders.append(['Cookie', 'c=c'])
        lureq = self.create_request(self.host, None)
        opener.open(lureq)
        cookie.save(ignore_discard=True, ignore_expires=True)

    # 加载cookie
    def load_cookie(self):
        self.cookie = cookielib.MozillaCookieJar().load(self.cookie_dir, ignore_discard=True, ignore_expires=True)

    # 设置cookie
    def set_cookie(self, key_values):
        self.opener.addheaders.append(['Cookie', key_values])

    # 加载opener
    def load_opener(self):
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookie))

    # 创建网络请求
    def create_request(self, url, params):
        data = None
        if params is not None:
            data = urllib.urlencode(params)
        request = urllib2.Request(url, data, self.headers)
        return request

    # 访问url
    def visit_url(self, url, params):
        return self.opener.open(self.create_request(url, params))


class GGTranslator(object):
    def __init__(self):
        # 初始化网络访问工具
        self.opener = NetOpener("https://translate.google.cn/", "gg_cookie.txt");
        self.opener.init_cookie()
        self.opener.load_cookie()
        self.opener.load_opener()
        # 加载 Google 请求识别标识生成方法
        self.gg_tk_js = execjs.compile("""
            var b = function (a, b) {
                for (var d = 0; d < b.length - 2; d += 3) {
                    var c = b.charAt(d + 2),
                        c = "a" <= c ? c.charCodeAt(0) - 87 : Number(c),
                        c = "+" == b.charAt(d + 1) ? a >>> c : a << c;
                    a = "+" == b.charAt(d) ? a + c & 4294967295 : a ^ c
                }
                return a
            }

            var tk =  function (a,TKK) {
                //console.log(a,TKK);
                for (var e = TKK.split("."), h = Number(e[0]) || 0, g = [], d = 0, f = 0; f < a.length; f++) {
                    var c = a.charCodeAt(f);
                    128 > c ? g[d++] = c : (2048 > c ? g[d++] = c >> 6 | 192 : (55296 == (c & 64512) && f + 1 < a.length && 56320 == (a.charCodeAt(f + 1) & 64512) ? (c = 65536 + ((c & 1023) << 10) + (a.charCodeAt(++f) & 1023), g[d++] = c >> 18 | 240, g[d++] = c >> 12 & 63 | 128) : g[d++] = c >> 12 | 224, g[d++] = c >> 6 & 63 | 128), g[d++] = c & 63 | 128)
                }
                a = h;
                for (d = 0; d < g.length; d++) a += g[d], a = b(a, "+-a^+6");
                a = b(a, "+-3^+b+-f");
                a ^= Number(e[1]) || 0;
                0 > a && (a = (a & 2147483647) + 2147483648);
                a %= 1E6;
                return a.toString() + "." + (a ^ h)
            }
        """)
        self.tkk = ""
        self.jsondecoder = JSONDecoder()

    # 获取识别标识生成参数 tkk
    def init_tkk(self):
        page_html = self.opener.visit_url("https://translate.google.cn/", None)
        page_str = page_html.read()
        tkk_str = re.findall(r'TKK=(eval\(\'[^\']*\'\))', page_str)[0]
        tkk_mtd = "".join(["var tkk = function () { return ", tkk_str, "}"])
        tkk_js = execjs.compile(tkk_mtd)
        self.tkk = tkk_js.call("tkk")

    # 生成请求识别标识
    def get_tk(self, source):
        return self.gg_tk_js.call("tk", source, self.tkk)

    # 翻译英文为中文的实现方法，提供给外部调用
    def en_to_zh(self, source):
        self.init_tkk()
        tk = self.get_tk(source)
        params = {"tk": tk, "q": source}
        # 翻译接口连接
        url = "https://translate.google.cn/translate_a/single?client=t&sl=en&tl=zh-CN&hl=zh-CN&dt=at&dt=bd&dt=ex" \
              "&dt=ld&dt=md&dt=qca&dt=rw&dt=rm&dt=ss&dt=t&ie=UTF-8&oe=UTF-8&otf=1&ssel=0&tsel=0&kc=5" \
              "&" + urllib.urlencode(params)
        # 翻译结果
        rsp = self.opener.visit_url(url, None).read()
        rjson = self.jsondecoder.decode(rsp)
        results = rjson[0]
        rarr = []
        # 解析结果
        for ritem in results:
            ritem_str = ritem[0]
            if ritem_str and ritem_str != "":
                rarr.append(ritem_str.strip().encode("utf-8"))
        return "\n".join(rarr)


translator = GGTranslator()

if __name__ == "__main__":
    ggt = GGTranslator()
    test_html = open("data/test.txt", "w")
    test_html.write(ggt.en_to_zh("""
    Jacket by b.Young Wool-mix knit Fluffy finish Funnel neck Button placket Functional pockets Regular fit - true to size  Machine wash 70% Polyester, 30% Wool Our model wears a UK S/EU S/US XS and is 168cm/5'6" tall
    Wool-mix knit 
    Fluffy finish 
    Funnel neck 
    Button placket 
    Functional pockets 
    Regular fit - true to size 
    Machine wash
    70% Polyester, 30% Wool
    Our model wears a UK S/EU S/US XS and is 168cm/5'6" tall
    """))



# -*- coding: UTF-8 -*-
import oss2
import time
import re
import requests
from repository import dao
from modules.repository import repositories
from modules.scheduler import task_scheduler
from mozia.crawler.tools.oss_utils import download_file_from_images_media
from mozia.crawler.tools import save_file_to_ftp, TRANSFER_COVER_DIR, PROCESS_COVER_DIR, DOWNLOAD_COVER_DIR
import os

endpoint = "oss-cn-shanghai.aliyuncs.com";
access_key_id = "Ik486j07fnzdJxVC";
access_key_secret = "zv8ESNdIrZ7wv2CZOeEMEgJhiDIoKC";
bucket_name = "images-media";
aliasUrl = "http://cdn.fashionfinger.com";

# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)


def download_and_retry(url, times=3):
    try:
        return requests.get(url)
    except Exception as e:
        print("download error, retry...", repr(e))
        if times > 0:
            return download_and_retry(url, times - 1)


def save_image_to_oss(task, image_url, index):
    if is_oss_url(image_url):
        return image_url

    print("image url:", image_url)
    oss_url = "s%02d/%04d/%08d/%02d.jpg" % (task["source_type"], task["designer_id"], task["product_id"], index)
    page = download_and_retry(image_url)
    result = bucket.put_object(oss_url, page.content)
    if 200 != result.status:
        raise Exception("save to oss error", result.status)

    return "https://images-media.oss-cn-shanghai.aliyuncs.com/%s" % oss_url


def is_oss_url(image_url):
    if not image_url:
        print("invalid image url:", image_url)
        return False

    pattern = re.compile("http(s)?://images-media\.oss-cn-shanghai\.aliyuncs\.com/.*")
    match = pattern.match(image_url)
    return match


# 下载图片并设置商品图片
def download_images(task):
    print("download images:", task)
    image_urls = dao.platform.get_product_sku_images(task["product_id"])
    if not image_urls or len(image_urls) == 0:
        print("[%s]no images download" % task["product_id"])
        return

    oss_urls = [
        save_image_to_oss(task, image_url, index)
        for (index, image_url) in enumerate(image_urls)
        if image_url
    ]
    save_product_images(task["product_id"], oss_urls)
    # save_cover_to_dir(task, cover)


def save_product_images(product_id, oss_urls):
    cover_url = "%s/cover.jpg" % os.path.dirname(oss_urls[0])
    dao.platform.save_product_images(product_id, cover_url, oss_urls)
    # 封面图片已从源网站下载到阿里云，等待P图人员处理
    dao.task.set_product_task_status(product_id, 1, oss_urls[0])


def add_ftp_cover_task(cover_task):
    return dao.task.add_ftp_cover_task(cover_task)


def save_cover_to_dir(task, cover_url):
    cover_dir = "%s/%s" % (DOWNLOAD_COVER_DIR, time.strftime('%Y%m%d', time.localtime(time.time())))
    if not os.path.isdir(cover_dir):
        os.makedirs(cover_dir)
    path = cover_url.replace("https://images-media.oss-cn-shanghai.aliyuncs.com/", "")
    cover_path = "%s/%s" % (cover_dir, "-".join(path.split("/")))
    print("save cover:", cover_path)
    page = download_and_retry(cover_url)
    cover_file = open(cover_path, "wb")
    cover_file.write(page.content)
    cover_file.close()


def download_oss_cover(cover_url):
    if not cover_url:
        print("Invalid cover url:", cover_url)
        return
    print("download oss cover:", cover_url)

    local_dir = TRANSFER_COVER_DIR
    if not os.path.isdir(local_dir) and not os.path.islink(local_dir):
        os.makedirs(local_dir)

    pattern = re.compile("https://images-media.oss-cn-shanghai.aliyuncs.com/(.*)")
    match = pattern.match(cover_url)
    if match:
        oss_path = match.group(1)
        file_name = oss_path.replace("/", "-")
        local_path = "%s/%s" % (local_dir, file_name)
        # network.download_image(cover_url, local_path)
        download_file_from_images_media(oss_path, local_path)
        # save_file_to_ftp("cover/%s" % file_name, local_path)
    else:
        print("cover url no match:", cover_url)


def start_images_downloader(sleep_seconds=600):
    dao.task.create_product_tasks()
    task = dao.task.get_product_task()
    while task:
        download_images(task)
        task = dao.task.get_product_task()


def start_download_oss_covers(sleep_seconds=30):
    while True:
        task = dao.task.get_ftp_cover_task()
        while task:
            try:
                download_oss_cover(task["cover_url"])
            except Exception as e:
                print('save ftp cover error', repr(e))
            task = dao.task.get_ftp_cover_task()

        time_string = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        print("[%s]no ftp cover task, sleep %ss... " % (time_string, sleep_seconds))
        time.sleep(sleep_seconds)


def get_today_covers():
    today = time.strftime('%Y%m%d', time.localtime(time.time()))
    return get_daily_covers(today)


def get_daily_covers(covers_date):
    cover_dir = "%s/%s" % (PROCESS_COVER_DIR, covers_date)
    pattern = re.compile("s[0-9]{2}-[0-9]{4}-[0-9]{6,8}-[0-9]{2}\.jpg")
    if os.path.isdir(cover_dir):
        return [
            name.replace("-", "/")
            for name in os.listdir(cover_dir)
            if pattern.match(name)
        ]
    else:
        os.makedirs(cover_dir)
        return []


def get_cover_dirs():
    if os.path.isdir(PROCESS_COVER_DIR):
        return [
            name for name in os.listdir(PROCESS_COVER_DIR)
        ]
    else:
        return []


if __name__ == "__main__":
    repositories.connect()
    task_scheduler.connect()
    # start_download_oss_covers()
    start_images_downloader(10)



# -*- coding: UTF-8 -*-
import json
import re

from bs4 import BeautifulSoup
from mozia.crawler.repository import dao

from modules import network
from modules.repository import repositories
from mozia.crawler.proxy.resolver import FarfetchResolver

ERROR = {"status": 500, "message": "Farfetch server error"}
SOLD_OUT = {"status": 100, "message": "Product sold out"}
NO_FOUND = {"status": 101, "message": "Product no found"}
KEYWORD_MAPPED = {
    "HIPS": "臀围",
    "BUST": "胸围",
    "WAIST": "腰围",
    "HEIGHT": "身高",
    "MODEL": "模特",
    "HEEL": "鞋跟"
}

COLOR_MAPPED = {
    "BLACK": "黑色"
}


def replace_words(string):
    keyword = string.split(":")[0]
    value = KEYWORD_MAPPED.get(keyword.upper())
    if value:
        return string.replace(keyword, value)
    return string


def crawl_product_sizes(product, catalog):
    product_id = product["id"]
    store_id = product.get("storeId")
    category_id = product["categoryId"],
    manufacturer_id = product["manufacturerId"]
    language_id = catalog["language_id"]

    if language_id == 2:
        url = "/it/product/GetDetailState?productId=%s&storeId=%s&sizeId=&categoryId=%s&designerId=%s"
    else:
        url = "/it/product/GetDetailState?productId=%s&storeId=%s&sizeId=&categoryId=%s&designerId=%s"
    url = url % (product_id, store_id, category_id, manufacturer_id)

    sizes_string = network.download("https://www.farfetch.cn%s" % url)

    try:
        return json.loads(sizes_string)["SizesInformationViewModel"]["AvailableSizes"]
    except:
        return None


def get_product_size(sku, product):
    price = sku["PriceInfo"]
    return {
        "original_price": int(price["Price"]),
        "size": sku["Description"],
        "scale": sku["ScaleDescription"],
        "currency": "EURO",
        "color": product["color"],
        "label": sku["MessageLabel"]
    }


def get_product_sizes(product, catalog):
    return [
        get_product_size(size, product)
        for size in crawl_product_sizes(product, catalog)
    ]


# /cn/shopping/women/dolce-gabbana-bellucci--item-11845899.aspx?storeid=9306&from=listing
def get_product_gender(url):
    pos = url.find("women")
    return 2 if -1 == pos else 1


def get_product_text_strings(soup):
    if not soup:
        return None

    lines = []
    dt = soup.find("dt")
    while dt:
        dd = dt.find_next_sibling("dd")
        lines.append(dt.string.strip() + dd.string.strip() if dd else "")
        dt = dd.find_next_sibling("dt")
    return lines


def get_product_image_urls(soup):
    images = []
    soup = soup.find("ul", class_="sliderProduct js-sliderProduct js-sliderProductPage")
    for image in soup.find_all("img", itemprop="image"):
        image_url = image["src"]
        # 图片只是 70 换成 1000
        images.append(image_url[:-6] + "1000.jpg")
    return images


def get_product_constitute(soup):
    # data - tstid = "Content_Composition&amp;Care" >
    ingredients = soup.find("div", attrs={
        "data-tstid": re.compile("Content_Composition")
    })
    lines = get_product_text_strings(ingredients)
    return "\n".join(lines)


def get_product_color(product):
    # soup = soup.find("span", itemprop="color")
    color_string = product["color"]
    if not color_string:
        return None

    [color_word, color_code] = color_string.split(",")
    color = dao.catalog.get_product_color(color_code, 1)
    if not color:
        dao.catalog.save_product_color({
            "color_code": color_code,
            "description": color_word,
            "source_type": 1,
            "color_name": COLOR_MAPPED.get(color_word)
        })
        return color_word
    return color["name"]


def get_product_measures(soup):
    strings = []
    if soup.dl.p:
        strings.append(soup.dl.p.string.strip())
    if soup.dl:
        measure_strings = get_product_text_strings(soup.dl)
        strings += measure_strings

    return [replace_words(item) for item in strings]


# 尺码信息
def get_product_size_description(product, soup):
    content_size_soup = soup.find("div", attrs={"data-tstid": "Content_Size&Fit"})
    if not content_size_soup:
        return None

    size_soup = content_size_soup.find("div", class_="mb30")
    strings = []
    if size_soup:
        strings += [string for string in size_soup.stripped_strings]

    category = product['category']
    if category in ('鞋履', '包袋'):
        centimeters_soup = soup.find(id="centimeters_wrapper")
        if centimeters_soup:
            strings += get_product_measures(centimeters_soup)

        model_is_wearing_soup = content_size_soup.find("p", class_="model_is_wearing")
        if model_is_wearing_soup:
            strings.append("".join([x for x in model_is_wearing_soup.stripped_strings]))

    return "\n".join(strings)


def get_product_description(product, soup):
    description = product.get("description")
    constitute = get_product_constitute(soup)
    size_description = get_product_size_description(product, soup)
    descriptions = [item for item in [description, constitute, size_description] if item]
    return "\n".join(descriptions)


def get_product_label(soup):
    label_soup = soup.find(class_="productLabel")
    return label_soup.string.strip() if label_soup else None


def crawl_farfetch_product(catalog):
    html = network.download("https://www.farfetch.cn%s" % catalog["url"])
    if re.search("HTTP/1.0 500 Server Error", html):
        return ERROR

    soup = BeautifulSoup(html, "html.parser")

    product_for_zh = FarfetchResolver(soup).process()
    # 如果新版本，则还需要获取一次英文版本的数据
    if product_for_zh:
        product_for_zh['gender'] = get_product_gender(catalog["url"])
        is_monitor = catalog.get('is_monitor')

        # 监控的商品不检查英文
        if not is_monitor:
            url = "https://www.farfetch.cn/it/%s" % catalog["url"][3:]
            html = network.download(url)
            soup = BeautifulSoup(html, "html.parser")
            product_for_en = FarfetchResolver(soup).process()
            product_for_zh['sizes'] = product_for_en['sizes']

        return product_for_zh

    print("product is old version, continue:", catalog["url"])

    if soup.find("div", class_="soldOut color-red bold h4"):
        return SOLD_OUT

    pattern = re.compile("window.universal_variable.product\s*=\s*({.*});")
    script_soup = soup.find("script", text=pattern)
    if not script_soup or not script_soup.string:
        return NO_FOUND

    match = re.search(pattern, script_soup.string.strip())
    if not match:
        return NO_FOUND

    # product_string = script_soup.string.strip().replace("window.universal_variable.product =", "")
    product = json.loads(match.group(1).strip())
    product["color"] = get_product_color(product)
    label = get_product_label(soup)
    season = '18SS' if '新季' == label else None
    return {
        "name": product["name"],
        "label": label,
        "season": season,
        "gender": get_product_gender(catalog["url"]),
        "image_urls": get_product_image_urls(soup),
        "designer_name": product["designerName"],
        "designer_code": product["designerStyleId"],
        "source_id": catalog["source_id"],
        "source_type": catalog["source_type"],
        "categories": [product["category"], product["subcategory"]],
        "description": get_product_description(product, soup),
        "sizes": get_product_sizes(product, catalog),
        "context": product
    }


if __name__ == "__main__":
    repositories.connect()
    catalog = {
        "source_type": 1,
        "language_id": 1,
        "source_id": "12646485",
        "url": "/cn/shopping/men/off-white---item-12646485.aspx?storeid=9502&from=listing&tglmdl=1&rnkdmnly=1"
    }

    print (json.dumps(crawl_farfetch_product(catalog)).decode("unicode_escape"))



# -*- coding: UTF-8 -*-
import urllib
import os
import oss2
import json
from repository import dao
from task import get_download_images_task, create_download_images_tasks

endpoint = "http://oss-cn-beijing.aliyuncs.com	"
access_key_id = "Ik486j07fnzdJxVC"
access_key_secret = "zv8ESNdIrZ7wv2CZOeEMEgJhiDIoKC"
bucket_name = "intra-mirror"
aliasUrl = "http://cdn.fashionfinger.com"
# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)


# 内容不全重新下载
def download_until_succeed(url, filename):
    try:
        urllib.urlretrieve(url, filename)
    except urllib.ContentTooShortError:
        print 'Network conditions is not good.Reloading.', url
        download_until_succeed(url, filename)


def download_images(image_urls, product_id):
    local_dir = "data/images/%s" % (product_id)
    if not os.path.exists(local_dir):
        os.makedirs(local_dir)

    oss_urls = []
    for index, url in enumerate(image_urls):
        if url:
            print ">>>>>>>>>>>begin download image:", url
            names = url.split("/")
            name = names[len(names) - 1]
            path = "%s\\%s" % (local_dir, name)
            download_until_succeed(url, path)
            print ">>>>>>>>>>>end download image:", url
            oss_url = "intra_mirror/%s/%s" % (product_id, name)
            bucket.put_object_from_file(oss_url, path)
            oss_urls.append("http://intra-mirror.oss-cn-beijing.aliyuncs.com/%s" % oss_url)
    # update_image_status(dir, oss_urls, product_id, cursor)
    dao.catalog.save_catalog_images(local_dir, oss_urls, product_id)


def start_download(task):
    image_urls = task["image_urls"]
    if not image_urls:
        return
    try:
        download_images(json.loads(image_urls), task["product_id"])
    except Exception as e:
        print  e


if __name__ == "__main__":
    while True:
        task = get_download_images_task()
        while task:
            start_download(task)
            task = get_download_images_task()
        create_download_images_tasks()



import pymysql
import time
import json


class ProductMonitor:
    def __init__(self):
        self.connection = pymysql.connect(host='10.26.235.6',
                                          port=3066,
                                          user='dba',
                                          passwd='123456',
                                          db='monitor',
                                          cursorclass=pymysql.cursors.DictCursor,
                                          charset='utf8mb4')
        timestamp = int(time.time())
        self.file_no_sku = open("data/no_sku_product_%d.txt" % timestamp, "w")

    def __del__(self):
        self.file_no_sku.close()

    def create_sku_size(self):
        sql = "call create_sku_size()"
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
        self.connection.commit()

    def set_no_sku_product_off(self):
        sql = "SELECT * FROM v_no_sku_product"
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            no_sku_products = cursor.fetchall()
            self.file_no_sku.write(json.dumps(no_sku_products))

        sql = """
          UPDATE fashion_product.t_product a, v_no_sku_product b 
          SET a.product_status=0,modify_time = unix_timestamp()
          WHERE a.product_id=b.product_id
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
        self.connection.commit()

    def start(self):
        self.create_sku_size()
        self.set_no_sku_product_off()


if __name__ == "__main__":
    ProductMonitor().start()



import commands
import shutil
import os


def copy_file_to_dir(file_path, target_dir):
    if not os.path.isdir(target_dir):
        os.makedirs(target_dir)
    shutil.copy(file_path, target_dir)


def deploy_weboff(source):
    print source
    target_dir = "/home/admin/transfer/weboff/"
    copy_file_to_dir(source, target_dir)
    (status, output) = commands.getstatusoutput('/home/web/weboff/deploy.sh')
    return str(output)


def deploy_official(source):
    print source
    target_dir = "/home/admin/transfer/official/"
    copy_file_to_dir(source, target_dir)
    (status, output) = commands.getstatusoutput('/home/web/official-site/deploy.sh')
    return str(output)


def deploy_finger(source):
    print source
    target_dir = "/home/admin/transfer/finger/"
    copy_file_to_dir(source, target_dir)
    (status, output) = commands.getstatusoutput('/home/web/finger/deploy.sh')
    return str(output)


def deploy_webffo(source):
    print source
    copy_file_to_dir(source, "/home/admin/transfer/webffo/")
    (status, output) = commands.getstatusoutput('/home/webffo/scripts/deploy.sh')
    return str(output)


def deploy_webadm(source):
    print source
    copy_file_to_dir(source, "/home/admin/transfer/webadm/")
    (status, output) = commands.getstatusoutput('/home/webadm/scripts/deploy.sh')
    return str(output)


def restart_webadm():
    (status, output) = commands.getstatusoutput('su - webadm -c "/home/webadm/scripts/startup.sh stop"')
    return str(output)


def restart_webffo():
    (status, output) = commands.getstatusoutput('su - webffo -c "/home/webffo/scripts/startup.sh stop"')
    return str(output)


def get_tail_logs(log_path, lines):
    (status, output) = commands.getstatusoutput('tail -n%s %s' % (lines, log_path))
    return str(output)


def restart_weboff():
    return "success"



# -*- coding: UTF-8 -*-
from modules.repository import Repository
import logging
import os

logger = logging.getLogger(os.path.basename(__file__))


class MonitorRepository(Repository):
    def __init__(self):
        Repository.__init__(self)

    def set_platform_product_status(self, product_id, product_status):
        print("[%s]set product status=%s" % (product_id, product_status))
        sql = """
        UPDATE fashion_product.t_product SET product_status=%s,modify_time=unix_timestamp(),editor_name='monitor' WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (product_status, product_id))
            self.connection.commit()
            cursor.close()

    def get_platform_product(self, product_id):
        sql = """
        SELECT * FROM fashion_product.t_product WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            product = cursor.fetchone()

        if not product:
            return product

        sql = """
           SELECT * FROM fashion_product.t_product_sku WHERE product_id=%s AND sku_status='ON'
           """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            product["skus"] = cursor.fetchall()

        return product

    def get_last_crawl_time(self, product_id):
        sql = "SELECT * FROM monitor.t_product_monitor WHERE product_id=%s"
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            return cursor.fetchone()

    def update_crawl_time(self, monitor_task, status):
        sql = """
        REPLACE INTO `monitor`.`t_product_monitor` 
        (`product_id`, `source_type`, `source_id`, `status`, `date_modified`) 
        VALUES 
        (%s, %s, %s, %s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (
                monitor_task["product_id"],
                monitor_task["source_type"],
                monitor_task["resource_code"],
                status))
            self.connection.commit()

    def set_platform_product_sizes_status(self, product_id, size, sku_status):
        sql = """
        UPDATE fashion_product.t_product_sku SET 
            sku_status=%s,
            modify_time=unix_timestamp(),
            editor_name='monitor' 
        WHERE product_id=%s AND size=%s
        """
        print("[%s]set product size:%s status=%s" % (product_id, size, sku_status))
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (sku_status, product_id, size))
            self.connection.commit()
            cursor.close()

        sql = """
        REPLACE INTO `monitor`.`t_product_sku_monitor` 
            (`product_id`, `size`, `status`, `date_modified`) 
        VALUES 
            (%s, %s, %s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (product_id, size, 0 if "OFF" == sku_status else 0))
            self.connection.commit()
            cursor.close()

    def set_platform_product_size_quantity(self, product_id, size, quantity):
        sql = """
        UPDATE fashion_product.t_product_sku SET quantity=%s,modify_time=unix_timestamp(),editor_name='monitor' WHERE product_id=%s AND size=%s
        """
        print("[%s]set product sku:%s quantity=%s" % (product_id, size, quantity))
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (quantity, product_id, size))
            self.connection.commit()
            cursor.close()

    def get_monitor_products(self):
        sql = """
        SELECT
          a.product_id,
          b.spider_product_id,
          b.source_url,
          b.resource_code,
          b.flag
        FROM
            fashion_product.t_product a,
            fashion_product.t_spider_product b
        WHERE
            a.spider_product_id IS NOT NULL
        AND a.spider_product_id = b.spider_product_id
        AND a.product_status = 1
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def get_monitor_products_from_im(self):
        sql = """
        SELECT
            c.url,
            b.*
        FROM
            fashion_product.t_product a,
            spider.t_product_source b,
            spider.t_catalog c
        WHERE
            a.product_id = b.product_id
        AND b.source_type = 3
        AND b.source_id = c.source_id
        AND b.source_type = c.source_type
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()


class Dao:
    def __init__(self):
        self.monitor = None
        self.create_repositories()

    def create_repositories(self):
        self.monitor = MonitorRepository()

    def reset(self):
        self.create_repositories()


dao = Dao()



import requests
import json
import time
from repository import dao


def get_total_page():
    page = requests.get("http://106.14.205.107:8083/shoplus-buyer/Home/get/getProductList?pageNumber=0&pageSize=30")
    product_info = json.loads(page.content)
    product_list = product_info[u'productList']
    total_page = product_list[u'totalPage']
    return int(total_page)


def crawl_catalog():
    url = "http://106.14.205.107:8083/shoplus-buyer/Home/get/getProductList?pageNumber=%s&pageSize=30"
    for i in range(0, get_total_page()):
        print "start url:", url % i
        page = requests.get(url % i)
        product_info = json.loads(page.content.decode(encoding="utf-8"))
        product_list = product_info[u'productList']
        products = product_list[u'list']
        dao.catalog.save_catalogs(products)


if __name__ == "__main__":
    while True:
        try:
            crawl_catalog()
        except Exception as e:
            print e
        dao.catalog.reset()
        time.sleep(600)



# -*- coding: UTF-8 -*-
from ftplib import FTP
import os
import re
from oss_utils import put_file, put_file_to_images_media
import time
from ftp_client import get_ftp_client

oss_root = "ptncp"

DATA_DIR = "/home/develop/data"
TRANSFER_COVER_DIR = "%s/transfer_covers" % DATA_DIR
DOWNLOAD_COVER_DIR = "%s/download_covers" % DATA_DIR
PROCESS_COVER_DIR = "%s/process_covers" % DATA_DIR


def save_file_to_oss(local_path):
    base_name = os.path.basename(local_path)
    (name, ext) = os.path.splitext(base_name)
    oss_url = oss_root + "/" + name + "/cover" + ext
    return put_file(oss_url, local_path)


def save_file_to_local(ftp_file):
    ftp = get_ftp_client()
    print("save ftp file:", ftp_file)
    today = time.strftime('%Y%m%d', time.localtime(time.time()))
    local_dir = "%s/%s" % (PROCESS_COVER_DIR, today)
    if not os.path.exists(local_dir):
        os.makedirs(local_dir)

    path = "%s/%s" % (local_dir, os.path.basename(ftp_file))
    local_file = open(path, 'wb')  # 打开要保存文件
    ftp.retrbinary("RETR " + ftp_file, local_file.write)  # 保存FTP上的文件
    local_file.close()
    return path


def save_file_to_images_media(path):
    cover_name = os.path.basename(path)
    oss_url = cover_name.replace("-", "/")
    # print(os.path.dirname(oss_url))
    oss_url = "%s/cover.jpg" % os.path.dirname(oss_url)
    put_file_to_images_media(oss_url, path, True)


def is_images_media_cover(cover_path):
    cover_name = os.path.basename(cover_path)
    pattern = re.compile("s[0-9]{2}-[0-9]{4}-[0-9]{6,8}-[0-9]{2}\.jpg")
    return pattern.match(cover_name)


def upload_modified_covers():
    ftp = get_ftp_client()

    for ftp_file in ftp.nlst("cover_modified"):
        path = save_file_to_local(ftp_file)
        if is_images_media_cover(path):
            save_file_to_images_media(path)
        else:
            save_file_to_oss(path)

        ftp.rename(ftp_file, "cover_updated/" + os.path.basename(ftp_file))


def start_cover_uploader():
    while True:
        try:
            timestamp = int(time.time())
            print "[%s]start upload cover" % time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            upload_modified_covers()
            end_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            print "[%s]end upload cover, cost %d," % (end_time, (int(time.time()) - timestamp))
            time.sleep(10)
        except Exception as e:
            print repr(e)


if __name__ == "__main__":
    start_cover_uploader()



# -*- coding: UTF-8 -*-
import oss2

endpoint = "http://oss-cn-shenzhen.aliyuncs.com";
access_key_id = "Ik486j07fnzdJxVC";
access_key_secret = "zv8ESNdIrZ7wv2CZOeEMEgJhiDIoKC";
bucket_name = "ssfk-media01";
aliasUrl = "http://cdn.fashionfinger.com";

# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

endpoint2 = "oss-cn-shanghai.aliyuncs.com";
access_key_id2 = "Ik486j07fnzdJxVC";
access_key_secret2 = "zv8ESNdIrZ7wv2CZOeEMEgJhiDIoKC";
bucket_name2 = "images-media";
aliasUrl2 = "http://cdn.fashionfinger.com";

bucketImages = oss2.Bucket(oss2.Auth(access_key_id2, access_key_secret2), endpoint2, bucket_name2)


class PutObjectResult:
    def __init__(self):
        self.status = 200
        self.etag = "EXISTS"


def download_file_from_images_media(oss_url, local_path):
    bucketImages.get_object_to_file(oss_url, local_path)


def put_file(oss_path, file_path, is_rewrite=True):
    if not is_rewrite:
        exists = bucket.object_exists(oss_path)
        if exists:
            print "object exists:", exists
            return PutObjectResult()

    print "put file:", oss_path
    return bucket.put_object_from_file(oss_path, file_path)


def put_file_to_images_media(oss_path, file_path, is_rewrite=True):
    if not is_rewrite:
        exists = bucketImages.object_exists(oss_path)
        if exists:
            print "object exists:", exists
            return PutObjectResult()

    print "put file:", oss_path
    return bucketImages.put_object_from_file(oss_path, file_path)



# -*- coding: UTF-8 -*-
import json
import re
import sys

reload(sys)
sys.setdefaultencoding("utf-8")


class ProductConverter:
    def __init__(self, soup, product, sizes_string, language_id):
        self.soup = soup
        self.product = product
        self.language_id = language_id
        if sizes_string:
            sizes_wrapper = json.loads(sizes_string)["SizesInformationViewModel"]
            self.sizes = [self.get_product_size(size, sizes_wrapper["IsLowStock"]) for size in
                          sizes_wrapper["AvailableSizes"]]
        else:
            self.sizes = []

    @staticmethod
    def get_text_strings(soup):
        if not soup:
            return None

        lines = []
        dt = soup.find("dt")
        while dt:
            dd = dt.find_next_sibling("dd")
            lines.append(dt.string.strip() + dd.string.strip() if dd else "")
            dt = dd.find_next_sibling("dt")
        return lines

    def get_product_size(self, size_context, low_stock):
        price_info = size_context["PriceInfo"]
        size = {
            "size": size_context["Description"],
            "size_id": size_context["SizeId"],
            "store_id": size_context["StoreId"],
            "scale_id": size_context["ScaleId"],
            "market_price": price_info.get("Price"),
            "scale": size_context["ScaleDescription"]
        }

        if size_context["LastInStock"]:
            size["quantity"] = 1
        else:
            size["quantity"] = 0 if low_stock else None

        formated_price_without_promotion = price_info["FormatedPriceWithoutPromotion"]
        formated_price = price_info["FormatedPrice"]
        if formated_price_without_promotion:
            size["price"] = formated_price_without_promotion
            size["sale_price"] = formated_price
        else:
            size["price"] = formated_price
            size["sale_price"] = formated_price
        if 1 == self.language_id:
            size["currency"] = size["price"][:1]
            size["price"] = (size["price"][1:]).replace(",", "")
            size["sale_price"] = (size["sale_price"][1:]).replace(",", "")
        elif 3 == self.language_id or 2 == self.language_id:
            size["currency"] = size["price"][-1:]
            size["price"] = (size["price"][:-1]).replace(".", "").strip()
            size["sale_price"] = (size["sale_price"][:-1]).replace(".", "").strip()

        return size

    def get_product_sizes(self):
        return self.sizes

    # 如果全是英文需要翻译
    def description(self):
        description = self.soup.find("p", itemprop="description").string.strip()
        return description

    def color(self):
        soup = self.soup.find("span", itemprop="color")
        return soup.string.strip() if soup else None

    # 尺码信息
    def size_description(self):
        content_size_soup = self.soup.find("div", attrs={"data-tstid": "Content_Size&Fit"})
        if not content_size_soup:
            return None

        size_soup = content_size_soup.find("div", class_="mb30")
        strings = []
        if size_soup:
            strings += [string for string in size_soup.stripped_strings]

        measure_info_soup = content_size_soup.find("div", class_="measure-info")
        if measure_info_soup:
            for string in measure_info_soup.stripped_strings:
                strings.append(string)

        centimeters_soup = content_size_soup.find(id="centimeters_wrapper")
        if centimeters_soup:
            if centimeters_soup.dl.p:
                strings.append(centimeters_soup.dl.p.string.strip())
            if centimeters_soup.dl:
                measure_strings = self.get_text_strings(centimeters_soup.dl)
                strings += measure_strings

        model_is_wearing_soup = content_size_soup.find("p", class_="model_is_wearing")
        if model_is_wearing_soup:
            strings.append("".join([x for x in model_is_wearing_soup.stripped_strings]))

        return "\n".join(strings)

    def location(self):
        soup = self.soup.find("span", attrs={"data-tstid": "shippingCountryName"})
        if soup:
            return soup.string.strip()

    def designer_code(self):
        soup = self.soup.find("p", attrs={"data-tstid": "designerStyleId"})
        return soup.span.string.strip() if soup else None

    def category_id(self):
        return self.product["categoryId"]

    def manufacturer_id(self):
        return self.product["manufacturerId"]

    def designer_name(self):
        return self.product["designerName"]

    def price(self):
        return self.product["unitPrice"]

    def sale_price(self):
        return self.product["unitSalePrice"]

    def name(self):
        return self.product["name"]

    def product_id(self):
        return self.product["id"]

    def store_id(self):
        return self.product["storeId"]

    def currency(self):
        currency_code = self.product["currency"]
        if "EUR" == currency_code:
            return "EURO"
        else:
            return "RMB"

    def constitute(self):
        # data - tstid = "Content_Composition&amp;Care" >
        ingredients = self.soup.find("div", attrs={
            "data-tstid": re.compile("Content_Composition")
        })
        lines = self.get_text_strings(ingredients)
        return "\n".join(lines)

    def image_urls(self):
        images = []
        soup = self.soup.find("ul", class_="sliderProduct js-sliderProduct js-sliderProductPage")
        for image in soup.find_all("img", itemprop="image"):
            image_url = image["src"]
            # 图片只是 70 换成 1000
            images.append(image_url[:-6] + "1000.jpg")
        return images

    def gender(self):
        gender = self.product["gender"]
        if "MEN" == gender.upper():
            return 2
        elif "WOMEN" == gender.upper():
            return 1
        else:
            return 4

    def is_sold_out(self):
        sold_out_soup = self.soup.find("div", class_="soldOut color-red bold h4")
        if sold_out_soup:
            return sold_out_soup.prettify()

    def create(self):
        product = {
            "description": self.description(),
            "size_description": self.size_description(),
            "color": self.color(),
            "constitute": self.constitute(),
            "designer_code": self.designer_code(),
            "image_urls": self.image_urls(),
            "sizes": self.get_product_sizes(),
            "gender": self.gender(),
            "location": self.location(),
            "resource_code": self.product["product_id"],
            "designer": self.product["designerName"],
            "is_sold_out": self.is_sold_out(),
            "currency": self.currency(),
            "cover": self.product["imageUrl"]
        }
        return dict(self.product, **product)



# -*- coding: UTF-8 -*-
import json
from proxy import crawl_product
from flask import request
from flask import Flask
from flask import make_response
from flask_cors import CORS
from modules.tools.translate import translator
from modules.repository import repositories
from modules.scheduler import task_scheduler
from mozia.crawler.downloader import \
    get_today_covers, \
    get_daily_covers, \
    get_cover_dirs, \
    save_product_images, \
    add_ftp_cover_task

app = Flask(__name__)
CORS(app)

SUCCESS = {
    "status": 0,
    "message": "success"
}


@app.route('/proxy/product', methods=["POST", "GET"])
def get_product():
    catalog = {
        "source_type": 1,
        "catalog_id": 7909,
        "language_id": 1,
        "source_id": "12598893",
        "url": "/cn/shopping/women/mm6-maison-margiela-pleated-side-dress-item-12598893.aspx?storeid=9306&from=listing",
        "thumbnail": "https://cdn-images.farfetch-contents.com/12/58/75/59/12587559_12078291_255.jpg",
        "product_name": "leopard trim cropped trousers",
        "catalog_context": "{\"imageUrl\": \"https://cdn-images.farfetch-contents.com/12/58/75/59/12587559_12078291_255.jpg\", \"unit_sale_price\": 5617.1645, \"color\": \"\", \"CurrencyCode\": \"CNY\", \"name\": \"leopard trim cropped trousers\", \"skuCode\": \"12587559\", \"url\": \"/cn/shopping/women/dolce-gabbana-leopard-trim-cropped-trousers-item-12587559.aspx?storeid=9306&from=listing\", \"currencyCode\": \"CNY\", \"storeId\": 9306, \"sku_code\": \"12587559\", \"unit_price\": 5617.1645, \"currency\": \"CNY\", \"designerName\": \"Dolce & Gabbana\", \"image_url\": \"https://cdn-images.farfetch-contents.com/12/58/75/59/12587559_12078291_255.jpg\", \"stock\": 8, \"unitSalePrice\": 5617.1645, \"unitPrice\": 5617.1645, \"id\": \"12587559\", \"hasStock\": true, \"manufacturer\": \"Dolce & Gabbana\"}",
        "catalog_status": 0
    }

    if request.method == 'POST':
        product = crawl_product(request.get_json())
        if not product :
            product = {}
        response = make_response(json.dumps(product))
    else:
        response = make_response(json.dumps(crawl_product(catalog)))

    response.headers['Access-Control-Allow-Origin'] = '*'
    response.headers['Access-Control-Allow-Methods'] = 'POST'
    response.headers['Access-Control-Allow-Headers'] = 'x-requested-with,content-type'

    return response


@app.route('/translate', methods=["POST", "GET"])
def translate():
    if request.method == 'POST':
        string = translator.en_to_zh(request.get_json()["text"])
    else:
        string = translator.en_to_zh(
            "Jet Set Travel saffiano leather tote bag<br>Model in saffiano print calf leather<br />Central zip closure<br />2 flat handles with buckle<br />2 internal compartments and a central partition with zip<br />1 inside pocket with zip<br />3 inside flat pockets<br />1 pocket for mobilephone<br />1 removable logoed charm<br />Gold-tone hardware<br />Fabric lining with allover logo<br />Length: 38 cm<br />Height: 28 cm<br />Depth: 15 cm<br>Posizione del logo: Metallic logo on front<br>100%, Leather<br>Confezione: Acetate<br>Colore: black<br>Product Model: Jet Set Travel")
    response = make_response(string)
    print(string)
    return response


@app.route('/favicon.ico')
def favicon():
    return []


@app.route('/covers')
@app.route('/covers/<date_string>')
def covers(date_string=None):
    if not date_string:
        today_covers = get_today_covers()
    else:
        today_covers = get_daily_covers(date_string)

    data = {
        "covers": today_covers,
        "length": len(today_covers),
        "status": 0,
        "message": "success"
    }
    return make_response(json.dumps(data))


@app.route('/cover/dirs')
def covers_dir():
    dirs = get_cover_dirs()
    data = {
        "covers": dirs,
        "length": len(dirs),
        "status": 0,
        "message": "success"
    }
    return make_response(json.dumps(data))


@app.route('/cover/createFtpTask', methods=["POST", "GET"])
def ftp_cover():
    if request.method == 'POST':
        ftp_cover_task = request.get_json()
    else:
        ftp_cover_task = {
            "product_id": 0,
            "cover_url": "https://images-media.oss-cn-shanghai.aliyuncs.com/s01/0050/00000045/cover.jpg"
        }
    add_ftp_cover_task(ftp_cover_task)
    response = make_response(json.dumps(SUCCESS))
    return response


@app.route('/product/saveImages')
def set_product_images():
    data = request.get_json()
    save_product_images(data["product_id"], data["image_urls"])
    return make_response(json.dumps(data))


app.config['SECRET_KEY'] = 'off'
if __name__ == '__main__':
    repositories.connect()
    task_scheduler.connect()
    app.run(debug=False, port=9604, host="0.0.0.0")



import pymysql

connection = pymysql.connect(host='172.16.8.147',
                             port=3306,
                             user='dba',
                             passwd='123456',
                             db='greenbox',
                             cursorclass=pymysql.cursors.DictCursor,
                             charset='utf8mb4')



# -*- coding: UTF-8 -*-
import json
import time
import logging
import farfetch.network
import re
import os
import datetime
from bs4 import BeautifulSoup

logger = logging.getLogger(os.path.basename(__file__))

from farfetch.keys import KEY_MONITOR_PRODUCT_TASKS
from modules.scheduler import task_scheduler
from repository import dao
from modules.repository import repositories


def get_monitor_task_length():
    return task_scheduler.redis.llen(KEY_MONITOR_PRODUCT_TASKS)


def create_monitor_product_tasks():
    task_length = get_monitor_task_length()
    if task_length > 0:
        print "Have task in queue:", task_length
        return

    for monitor_task in dao.monitor.get_monitor_products():
        print "create monitor task:", monitor_task
        monitor_task["time"] = int(time.time())
        task_scheduler.push(KEY_MONITOR_PRODUCT_TASKS, json.dumps(monitor_task))


def get_monitor_product_task():
    task_length = get_monitor_task_length()
    print "tasks in queue:", task_length
    task_string = task_scheduler.pop(KEY_MONITOR_PRODUCT_TASKS)
    if not task_string:
        return None
    return json.loads(task_string)


def get_product_sizes_url(product_id, store_id, category_id, manufacturer_id):
    url = "https://www.farfetch.cn/it/product/GetDetailState?productId=%s&storeId=%s&sizeId=&categoryId=%s&designerId=%s"
    return url % (product_id, store_id, category_id, manufacturer_id)


def is_from_farfetch(monitor_task):
    source_url = monitor_task["source_url"]
    return source_url.startswith("https://www.farfetch.cn/")


def is_need_crawl(monitor_task):
    last_crawl = dao.monitor.get_last_crawl_time(monitor_task["product_id"])
    if not last_crawl:
        return True

    # 两小时内不重复爬
    date_modified = last_crawl["date_modified"]
    passed_seconds = ((datetime.datetime.now() - date_modified).total_seconds())
    return passed_seconds > 3600 * 2


def check_platform_product_sizes(monitor_task, source_sizes):
    product_id = monitor_task["product_id"]

    platform_product = dao.monitor.get_platform_product(product_id)
    if not platform_product:
        print("platform product no exists", product_id)
        return True

    platform_product_sizes = platform_product.get("skus")

    # 平台商品没有sku,直接下架
    if not platform_product_sizes or len(platform_product_sizes) == 0:
        print("platform product no sizes:", product_id)
        dao.monitor.set_platform_product_status(product_id, 0)

    source_sizes_mapped = {}
    for size in source_sizes:
        size_name = ("%s[%s]" % (size["Description"], size["ScaleDescription"])).upper()
        source_sizes_mapped[size_name] = size
        source_sizes_mapped[size["Description"].upper()] = size
        if "OS" == size["Description"]:
            source_sizes_mapped["One Size".upper()] = size

    close_size_number = 0
    for size in platform_product_sizes:
        if not source_sizes_mapped.get(size["size"].upper()):
            dao.monitor.set_platform_product_sizes_status(product_id, size["size"], "OFF")
            close_size_number += 1

    return len(platform_product_sizes) > close_size_number


def compare_size_from_crawl(sizes_object, monitor_task):
    # sizes_info = json.loads(sizes_string)["SizesInformationViewModel"]
    sizes = sizes_object["AvailableSizes"]
    if not sizes:
        print "[%s]product no available sizes" % monitor_task["product_id"], monitor_task["source_url"]
        dao.monitor.set_platform_product_status(monitor_task["product_id"], 0)
        dao.monitor.update_crawl_time(monitor_task, 0)
    else:
        # TODO: 这里需要处理尺码已下架的情况
        if check_platform_product_sizes(monitor_task, source_sizes=sizes):
            dao.monitor.update_crawl_time(monitor_task, 1)
        else:
            print "[%s]product all size closed" % monitor_task["product_id"], monitor_task["source_url"]
            dao.monitor.update_crawl_time(monitor_task, 0)


def crawl_product_for_monitor(monitor_task):
    if not is_from_farfetch(monitor_task):
        return

    # if not is_need_crawl(monitor_task):
    #     return

    source_url = monitor_task["source_url"].replace("https://www.farfetch.cn/cn/", "https://www.farfetch.cn/it/")
    monitor_task["source_url"] = source_url
    monitor_task["source_type"] = "FARFETCH"
    timestamp = int(time.time())

    print "begin task:", json.dumps(monitor_task)
    page = farfetch.network.download(source_url)

    if re.search("HTTP/1.0 500 Server Error", page):
        print("farfetch server error, sleep 5s")
        time.sleep(5)
        return

    soup = BeautifulSoup(page, "html.parser")

    sold_out_soup = soup.find("div", class_="soldOut color-red bold h4")
    if sold_out_soup:
        sold_out_string = sold_out_soup.string.strip()
        print("product sold out:", sold_out_string)
        dao.monitor.set_platform_product_status(monitor_task["product_id"], 0)
        dao.monitor.update_crawl_time(monitor_task, 0)
        return

    script_soup = soup.find("script", text=re.compile("window.universal_variable.product\s*=\s*"))

    # 找不到商品信息
    if not script_soup:
        # 商品已售罄
        print "product sold out:", json.dumps(monitor_task)
        dao.monitor.set_platform_product_status(monitor_task["product_id"], 0)
        return

    product = json.loads(
        script_soup.string.strip().replace("window.universal_variable.product =", "").strip().rstrip(";"))
    sizes_url = get_product_sizes_url(product["id"], product.get("storeId"), product["categoryId"],
                                      product["manufacturerId"])

    sizes_string = farfetch.network.download(sizes_url)
    if sizes_string:
        try:
            compare_size_from_crawl(json.loads(sizes_string)["SizesInformationViewModel"], monitor_task)
        except Exception:
            print "invalid sizes string:", sizes_string
            raise
    else:
        # 没有尺码则下架
        print "product no size info:", sizes_string
        dao.monitor.set_platform_product_status(monitor_task["product_id"], 0)
        dao.monitor.update_crawl_time(monitor_task, 0)

    print "end task, cost(%d):" % (int(time.time()) - timestamp), monitor_task["product_id"]


def start_farfetch_monitor():
    create_monitor_product_tasks()
    task = get_monitor_product_task()
    while task:
        crawl_product_for_monitor(task)
        task = get_monitor_product_task()


if __name__ == "__main__":
    task_scheduler.connect()
    repositories.connect()
    task = {"spider_product_id": 62287, "product_id": 67442,
            "source_url": "https://www.farfetch.cn/it/shopping/men/misbhv-logo--item-12426681.aspx?storeid=10230&amp;from=1",
            "source_type": "FARFETCH", "flag": "018", "time": 1514875442, "resource_code": "12426681"}

    crawl_product_for_monitor(task)



# -*- coding: UTF-8 -*-
# from bs4 import BeautifulSoup
import re
import json
from mozia.crawler.repository import dao

COLOR_MAPPED = {
    "BLACK": "黑色"
}


# return {
#     "name": product["name"],
#     "label": get_product_label(soup),
#     "gender": get_product_gender(catalog["url"]),
#     "image_urls": get_product_image_urls(soup),
#     "designer_name": product["designerName"],
#     "designer_code": product["designerStyleId"],
#     "source_id": catalog["source_id"],
#     "source_type": catalog["source_type"],
#     "categories": [product["category"], product["subcategory"]],
#     "description": get_product_description(product, soup),
#     "sizes": get_product_sizes(product, catalog),
#     "context": product
# }

# "designerDetails": {
#     "name": "Mm6 Maison Margiela",
#     "designerStyleId": "S52CT0312S25322",
#     "designerColour": "964 RUBBER/FUXIA",
#     "link": {
#         "url": "/cn/shopping/women/mm6-maison-margiela/items.aspx",
#         "text": "Mm6 Maison Margiela",
#         "trackingId": "pp_infobrd"
#     },
#     "id": 114084,
#     "description": "绝不在设计上妥协的MM6是Maison Martin Margiela的主要副线品牌。先将旧衣拆损，然后再将它们拼组成崭新的衣服，其创新精神在此独特的制衣手法得到完美的体现。"
# }

#  "composition": [
# {
#    "material": "成分:",
#    "value": "棉 100%"
# },
# {
#    "material": "成分:",
#    "value": "涤纶 100%"
# }
# ],
# "categories": {
# "135967": "服装",
# "135979": "连衣裙",
# "136189": "OL连衣裙"
# },


# "sizes": {
#     "available": {
#         "19": {
#             "lastInStock": false,
#             "storeId": 9306,
#             "sizeId": 19,
#             "description": "XS",
#             "quantity": 2
#         },
#         "20": {
#             "lastInStock": true,
#             "storeId": 9306,
#             "sizeId": 20,
#             "description": "S",
#             "quantity": 1
#         }
#     },
#     "selectedSize": null,
#     "friendlyScaleName": "",
#     "scaleId": 115,
#     "scaleDescription": "成衣均码",
#     "isOneSize": false
# }

class FarfetchResolver():
    def __init__(self, soup):
        self.soup = soup
        self.product = None

    # /cn/shopping/women/dolce-gabbana-bellucci--item-11845899.aspx?storeid=9306&from=listing
    def get_product_gender(self, url):
        pos = url.find("women")
        return 2 if -1 == pos else 1

    def get_product_color(self):
        colors_string = self.product['details']['colors']
        if not colors_string:
            return None

        [color_word, color_code] = colors_string.split(",")
        color = dao.catalog.get_product_color(color_code, 1)
        if not color:
            dao.catalog.save_product_color({
                "color_code": color_code,
                "description": color_word,
                "source_type": 1,
                "color_name": COLOR_MAPPED.get(color_word)
            })
            return color_word
        return color["name"]

    def get_product_size(self, sku, color):
        price_info = self.product["priceInfo"]["default"]
        return {
            "original_price": int(price_info["initialPrice"]),
            "size": sku["description"],
            "currency": "EURO",
            "color": color,
            "label": sku["quantity"]
        }

    def get_product_sizes(self, color):
        sizes = self.product['sizes']['available']
        return [
            self.get_product_size(size, color)
            for size in sizes.values()
        ]

    def get_designer_name(self):
        designer_details = self.product['designerDetails']
        return designer_details['name']

    def get_designer_code(self):
        designer_details = self.product['designerDetails']
        return designer_details['designerStyleId']

    def get_compositions(self):
        compositions = self.product['composition']
        items = []
        for item in compositions:
            items.append("%s%s" % (item['material'], item['value']))
        return items

    def get_measurements(self):
        measurements = self.product['measurements']
        if not measurements:
            return []

        if not measurements.get('modelMeasurements'):
            return []

        items = ["模特"]
        for (key, value) in measurements['modelMeasurements'].items():
            items.append("%s:%s" % (key, value[0]))
        return items

    def get_product_description(self):
        product_details = self.product['details']
        items = [product_details['description']] + self.get_measurements() + self.get_compositions()
        return "\n".join(items)

    def get_product_details(self):
        product_details = self.product['details']
        season = None
        if product_details.get('merchandiseTag'):
            tag = product_details['merchandiseTag']
            season = '18SS' if '新季' == tag else None

        return {
            'name': product_details['shortDescription'],
            'description': self.get_product_description(),
            'source_id': product_details['productId'],
            'source_type': 1,
            'season': season
        }

    # "categories": {
    #     "135967": "服装",
    #     "135979": "连衣裙",
    #     "136189": "OL连衣裙"
    # }
    def get_categories(self):
        categories = self.product['categories']
        items = []
        for category_id in sorted(categories.keys()):
            items.append(categories[category_id])
        # return categories.values()
        return items

    def get_image_urls(self):
        images_main = self.product['images']['main']
        image_urls = []
        for item in images_main:
            image_urls.append(item['zoom'])
        return image_urls

    def process(self):
        pattern = re.compile("window\['__initialState_slice-pdp__'\]\s*=\s*(.*);")
        product_script_soup = self.soup.find("script", text=pattern)

        # 找不到商品代码
        if not product_script_soup:
            return

        match = pattern.match(product_script_soup.string)
        if not match:
            return

        json_object = json.loads(match.group(1))
        self.product = json_object['productViewModel']
        # print(json.dumps(self.product).decode("unicode_escape"))

        color = self.get_product_color()
        product_object = {
            'measurements': self.get_measurements(),
            'designer_name': self.get_designer_name(),
            'designer_code': self.get_designer_code(),
            'compositions': self.get_compositions(),
            'categories': self.get_categories(),
            'image_urls': self.get_image_urls(),
            'sizes': self.get_product_sizes(color)
        }
        product_object.update(self.get_product_details())
        return product_object



# -*- coding: UTF-8 -*-
import json
import re
import time
from modules import network
from bs4 import BeautifulSoup
from repository import dao
from modules.repository import repositories


class IntramirrorCatalogCrawler:
    def __init__(self, task):
        self.task = task

    def start(self):
        url = self.task["url"]
        products = json.loads(network.download(url))
        self.create_catalogs(products["productList"])

    @staticmethod
    def get_product_url(catalog):
        return "http://106.14.205.107:8083/shoplus-buyer/api/product?id=%s" % catalog["shop_product_id"]

    @staticmethod
    def get_product_name(catalog):
        if catalog.get("skuList"):
            return catalog["skuList"][0]["name"]
            # skus = catalog["skuList"]

    @staticmethod
    def get_product_cover(catalog):
        if catalog.get("cover_img"):
            return json.loads(catalog["cover_img"])[0]

    @staticmethod
    def get_product_quantity(product):
        skus = product.get('skuList')
        if not skus:
            return 0
        quantity = 0
        for sku in skus:
            quantity += sku['store'] if sku.get('store') else 0
        return quantity

    def create_catalog(self, product):
        catalog = {
            "source_type": 3,
            "url": self.get_product_url(product),
            "source_id": product["shop_product_id"],
            "catalog_context": json.dumps(product),
            "catalog_status": 0,
            "product_name": self.get_product_name(product),
            "thumbnail": self.get_product_cover(product),
            "language_id": self.task["language_id"],
            "designer_name": product['brand_englishName'],
            "season": product['season_code'],
            "quantity": self.get_product_quantity(product)
        }
        # product["extend"] = extend
        dao.catalog.save_catalog(catalog)

    def create_catalogs(self, products):
        # print(json.dumps(products))
        for product in products["list"]:
            try:
                self.create_catalog(product)
            except Exception as e:
                print(repr(e))


class FarfetchCatalogCrawler:
    def __init__(self, task):
        self.task = task

    @staticmethod
    def get_total_page(soup):
        pagination = soup.find("li", class_="pagination-label")
        total_page = pagination.find("span", attrs={
            "data-tstid": "paginationTotal"
        }).string
        return int(total_page)

    def create_catalogs(self, soup):
        pattern = re.compile("window.universal_variable.listing\s*=\s*({.*});")
        # soup = soup.find(pattern)
        script_soup = soup.find("script", text=pattern)
        match = re.search(pattern, script_soup.string.strip())
        products = json.loads(match.group(1).strip())

        # products = json.loads(soup.string.strip().replace("window.universal_variable.listing =", "").rstrip(";"))
        # extend = json.loads(self.task.get("extend") or "{}")
        # extend["catalog_id"] = products.get("categoryId")

        for product in products["items"]:
            catalog = {
                "source_type": 1,
                "url": product["url"],
                "source_id": product["id"],
                "store_id": product["storeId"],
                "catalog_context": json.dumps(product),
                "catalog_status": 0,
                "product_name": product["name"],
                "thumbnail": product["imageUrl"],
                "language_id": self.task["language_id"],
                "designer_name": product["designerName"],
                "quantity": product["stock"],
                "season": None
            }
            # product["extend"] = extend
            dao.catalog.save_catalog(catalog)

    def crawl(self, page_url, task):
        print("begin url:", page_url)
        html = network.download(page_url)
        soup = BeautifulSoup(html, "html.parser")
        self.create_catalogs(soup)
        print("end url:", page_url)
        return soup

    def start(self):
        task = self.task
        url = task["url"]
        soup = self.crawl(url, task)
        quo = url.find("?")
        total_page = self.get_total_page(soup)

        print("total page(%d):" % total_page, url)
        for i in range(2, total_page):
            page_url = "%s&page=%d" % (url, i) if quo > 0 else "%s?page=%d" % (url, i)
            self.crawl(page_url, task)

            # dao.task.update_task_status(task)
            # try:
            #
            # except Exception as e:
            #     print('repr(e):', repr(e))


def crawl_intramirror_catalogs():
    task = {
        "url": "http://106.14.205.107:8083/shoplus-buyer/Home/get/getProductList?pageNumber=0&pageSize=6000",
        "language_id": 1
    }
    IntramirrorCatalogCrawler(task).start()


def crawl_farfetch_catalogs():
    url_string = """
https://www.farfetch.cn/cn/shopping/women/julian-fashion/items.aspx
https://www.farfetch.cn/cn/shopping/men/julian-fashion/items.aspx
https://www.farfetch.cn/cn/shopping/women/ratti/items.aspx
https://www.farfetch.cn/cn/shopping/men/ratti/items.aspx
https://www.farfetch.cn/cn/shopping/women/Luisa-Boutique/items.aspx
https://www.farfetch.cn/cn/shopping/men/Luisa-Boutique/items.aspx
https://www.farfetch.cn/cn/shopping/women/pozzilei/items.aspx
https://www.farfetch.cn/cn/shopping/men/pozzilei/items.aspx
https://www.farfetch.cn/cn/shopping/women/tessabit/items.aspx
https://www.farfetch.cn/cn/shopping/men/tessabit/items.aspx
https://www.farfetch.cn/cn/shopping/women/tiziana-fausti/items.aspx
https://www.farfetch.cn/cn/shopping/men/tiziana-fausti/items.aspx
https://www.farfetch.cn/cn/shopping/women/Base-blu/items.aspx
https://www.farfetch.cn/cn/shopping/men/Base-blu/items.aspx
https://www.farfetch.cn/cn/shopping/women/Boutique-Sugar/items.aspx
https://www.farfetch.cn/cn/shopping/men/Boutique-Sugar/items.aspx
https://www.farfetch.cn/cn/shopping/women/voga/items.aspx
https://www.farfetch.cn/cn/shopping/men/voga/items.aspx
https://www.farfetch.cn/cn/shopping/women/vinicio/items.aspx
https://www.farfetch.cn/cn/shopping/men/vinicio/items.aspx
https://www.farfetch.cn/cn/shopping/women/Wise-Boutique/items.aspx
https://www.farfetch.cn/cn/shopping/men/Wise-Boutique/items.aspx
https://www.farfetch.cn/cn/shopping/women/fiacchini/items.aspx
https://www.farfetch.cn/cn/shopping/men/fiacchini/items.aspx
https://www.farfetch.cn/cn/shopping/women/mantovani/items.aspx
https://www.farfetch.cn/cn/shopping/men/mantovani/items.aspx
https://www.farfetch.cn/cn/shopping/women/stefania-mode/items.aspx
https://www.farfetch.cn/cn/shopping/men/stefania-mode/items.aspx
https://www.farfetch.cn/cn/shopping/women/coltorti/items.aspx
https://www.farfetch.cn/cn/shopping/men/coltorti/items.aspx
https://www.farfetch.cn/cn/shopping/women/boutique-parisi/items.aspx
https://www.farfetch.cn/cn/shopping/men/boutique-parisi/items.aspx
https://www.farfetch.cn/cn/shopping/women/papini/items.aspx
https://www.farfetch.cn/cn/shopping/men/papini/items.aspx"""

    tasks = [{"url": url.strip(), "language_id": 1} for url in url_string.split("\n") if url.strip()]
    for task in tasks:
        print(task)
        FarfetchCatalogCrawler(task).start()


if __name__ == "__main__":
    repositories.connect()
    crawl_intramirror_catalogs()
    # crawl_farfetch_catalogs()



# -*- coding: UTF-8 -*-
import re
from modules.scheduler import task_scheduler
from modules.repository import repositories
from mozia.crawler.repository import dao
from mozia.crawler.proxy import crawl_product


class ProductMonitor:
    def __init__(self, source_type=1):
        self.name = ''
        if not dao.task.get_monitor_task_length() > 0:
            dao.task.create_monitor_tasks(source_type)

    def start(self):
        task = dao.task.get_monitor_task()
        while task:
            try:
                task['is_monitor'] = True
                task['language_id'] = 1
                source = crawl_product(task)
                self.check_product(task, source)
                task = dao.task.get_monitor_task()
            except Exception as e:
                print("monitor error:", repr(e))

    @staticmethod
    def check_product_sold_out(task, source_product):
        if not source_product:
            return False

        status = source_product.get('status')
        # 商品已售罄
        if 100 == status or 101 == status:
            dao.monitor.set_platform_product_status(task['product_id'], 0)
            return False

        sizes = source_product['sizes']
        if not sizes or len(sizes) == 0:
            dao.monitor.set_platform_product_status(task['product_id'], 0)
            return False

        return True

    def check_product(self, task, source_product):
        # print(task, source)
        if not self.check_product_sold_out(task, source_product):
            print('product sold out,', task)
            dao.monitor.update_monitored_time(task, 0)
            return
        status = 1 if self.check_product_sizes(task, source_product) else 0
        # 下架商品
        if 0 == status:
            dao.monitor.set_platform_product_status(task['product_id'], 0)

        # 更新监控时间
        dao.monitor.update_monitored_time(task, status)

    @staticmethod
    def get_sizes_mapped(source):
        sizes_mapped = {}
        for size in source['sizes']:
            sizes_mapped[size['size']] = size
            if '均码' == size['size']:
                sizes_mapped['OS'] = size
        return sizes_mapped

    @staticmethod
    def get_size_key(size):
        pattern = re.compile('^(\w+|\d+)\[.*\]$')
        match = pattern.match(size)
        if match:
            return match.group(1)

        if size.upper() == 'ONE SIZE':
            return '均码'

        if size.upper() == 'ONESIZE':
            return '均码'

        return size

    def check_product_sizes(self, task, source):
        platform_product = dao.monitor.get_platform_product(task['product_id'])
        if not platform_product:
            print("product no found:", task)
            return True

        sizes_mapped = self.get_sizes_mapped(source)
        product_id = task['product_id']
        source_type = task['source_type']

        sizes_no_found = 0
        is_need_update = False
        for sku in platform_product['skus']:
            size_key = self.get_size_key(sku['size'])
            print('[%s:%s]check product size:%s => %s' % (product_id, source_type, size_key, sku['size']))
            sku_matched = sizes_mapped.get(size_key)
            if not sku_matched:
                dao.monitor.set_platform_product_sizes_status(product_id, sku['size'], 'OFF')
                sizes_no_found += 1
                is_need_update = True
            else:
                # 如果源网站的价格变化了，则平台商品的价格也跟着变化
                f_supply_price = sku_matched.get('supply_price')
                if f_supply_price:
                    f_supply_price = round(float(f_supply_price), 2)

                f_original_price = sku_matched.get('original_price')
                if f_original_price:
                    f_original_price = round(float(f_original_price), 2)

                f_old_supply_price = sku.get('supply_price')
                if f_old_supply_price:
                    f_old_supply_price = float(f_old_supply_price)

                f_old_original_price = sku.get('original_price')
                if f_old_original_price:
                    f_old_original_price = float(f_old_original_price)
                item = {
                    'size': sku_matched.get('size'),
                    'product_sku_id': sku.get('product_sku_id'),
                    'original_price': f_original_price,
                    'supply_price': f_supply_price,
                    'old_supply_price': f_old_supply_price,
                    'old_original_price': f_old_original_price,
                    'quantity': sku_matched['quantity'] if sku_matched.get('quantity') else 0
                }
                if f_supply_price != f_old_supply_price or f_original_price != f_old_original_price:
                    if source_type == 1:
                        # farfetch 商品只有当源商品价格变高了,平台才跟着变
                        if f_original_price and f_old_original_price and f_original_price > f_old_original_price:
                            dao.monitor.update_platform_product_sku(product_id, item)
                        else:
                            print('farfetch product price is lower:', item)
                    else:
                        dao.monitor.update_platform_product_sku(product_id, item)
                    is_need_update = True
                else:
                    print('price no change:', item)
        # 如果sku更新了，则需要更新商品修改时间，否则 es扫描不到
        if is_need_update:
            dao.monitor.update_platform_product_time(product_id)

        return len(platform_product['skus']) > sizes_no_found


if __name__ == "__main__":
    task_scheduler.connect()
    repositories.connect(host='rm-bp1ri4cfj25gqey09no.mysql.rds.aliyuncs.com', passwd='dba@123456')
    monitor = ProductMonitor()
    monitor.start()
    assert ('40' == monitor.get_size_key('40[IT]'))
    assert ('40' == monitor.get_size_key('40'))
    assert ('均码' == monitor.get_size_key('One Size'))
    assert ('均码' == monitor.get_size_key('OneSize'))
    assert ('均码' == monitor.get_size_key('One size'))
    assert ('44' == monitor.get_size_key('44[IT/FR]'))
    assert ('XL' == monitor.get_size_key('XL[IT]'))



# -*- coding: UTF-8 -*-
import json
import requests
import random
import time
from repository import dao


def crawl_product_from_monitor(product_id, url):
    page = requests.get(url)
    return json.loads(page.content.decode(encoding="utf-8"))


def crawl_product(catalog):
    url = "http://106.14.205.107:8083/shoplus-buyer/api/product?id=%s" % catalog["shop_product_id"]
    page = requests.get(url)
    product_string = page.content.decode(encoding="utf-8")
    product = json.loads(product_string)
    if not product.get("productStatus"):
        print "invalid product:", product, url
        dao.catalog.set_catalog_status(catalog["product_id"], 0)
        return None

    product["context_string"] = json.dumps(product)
    product["product_id"] = catalog["product_id"]
    product["category_id"] = product["categoryId"]
    product["designer_id"] = product["brandId"]
    product["color_code"] = product["colorCode"]
    product["product_status"] = product["productStatus"]
    product["designer_code"] = product["brandCode"]
    product["sale_price"] = product.get("salePrice")
    product["price"] = product.get("price")
    image_urls = product.get("pics")
    product["image_urls"] = json.dumps(image_urls) if image_urls else None
    thumbnails = product.get("thumbnails")
    product["thumbnails"] = json.dumps(thumbnails) if thumbnails else None
    product["skus"] = None
    dao.product.save_product(product)

    product["composition"] = product.get("composition")
    product["product_name"] = product["name"]
    product["location"] = product.get("madeIn")
    product["edd_description"] = product.get("edd_desc")
    product["edd_title"] = product.get("edd_title")
    dao.product.save_description(product)

    product["designer_name"] = product["brand"]
    dao.product.save_designer(product)
    return product


if __name__ == "__main__":
    while True:
        timestamp = int(time.time())
        print "begin crawl products", time.time()
        try:
            for catalog in dao.catalog.find_catalogs_for_task():
                crawl_product(catalog)
                # time.sleep(random.randint(1, 5))
        except Exception as e:
            print "crawl error", e
        print "end crawl products, cost:", int(time.time()) - timestamp
        # 重置连接
        dao.catalog.reset()
        time.sleep(random.randint(10, 15))



import json
from datetime import date, datetime
import decimal


class DatetimeEncoder(json.JSONEncoder):
    def default(self, obj):
        # if isinstance(obj, datetime.datetime):
        #     return int(mktime(obj.timetuple()))
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        elif isinstance(obj, decimal.Decimal):
            return float(obj)
        else:
            return json.JSONEncoder.default(self, obj)



# -*- coding: UTF-8 -*-
import urllib
import json
import pymysql
import sys


def saveError(key, error, refer, cursor):
    print "saveLog:", key, error
    sql = """
    INSERT INTO `test`.`t_intra_mirror_error` (`key`, `error`, `referer`, `date_added`, `date_modified`) VALUES 
    (%s, %s, %s, now(), now());
    """
    try:
        cursor.execute(sql, (key, str(error), str(refer)))
    except Exception as e:
        print e


def downloadImages(image_urls, product_id):
    for index, image_url in image_urls:
        print image_url, product_id


def checkProductExists(product_id, cursor):
    sql = """
    select product_id from test.t_intra_mirror where product_id=%s
    """
    try:
        cursor.execute(sql, (product_id))
        return cursor.fetchone()
    except Exception as e:
        print e
    return True


def checkSkuExists(product_sku_id, cursor):
    sql = """
    select product_sku_id from test.t_intra_mirror_skus where product_sku_id=%s
    """
    try:
        cursor.execute(sql, (product_sku_id))
        return cursor.fetchone()
    except Exception as e:
        print e
    return True


def saveSkus(skus, cursor):
    for index, sku in enumerate(skus):
        product_sku_id = sku[u'sku_id']
        shop_product_sku_id = sku[u'shop_product_sku_id']
        price = sku[u'price']
        code = sku[u'sku_code']
        name = sku[u'name']
        quantity = sku[u'store'] or None
        product_id = sku[u'product_id']
        sale_price = sku[u'sale_price']
        size = sku[u'productValue']
        try:
            if checkSkuExists(product_sku_id, cursor):
                sql = """
                    UPDATE `test`.`t_intra_mirror_skus` SET  
                    `product_id`=%s, 
                    `name`=%s, 
                    `size`=%s, 
                    `price`=%s, 
                    `sale_price`=%s, 
                    `quantity`=%s,
                    `code`=%s, 
                    `shop_product_sku_id`=%s, 
                    `referer`=%s,
                    `date_modified`=now() WHERE (`product_sku_id`=%s)
                """
                cursor.execute(sql, (
                    product_id, name, size, price, sale_price, quantity, code, shop_product_sku_id,
                    json.dumps(sku), product_sku_id))
            else:
                sql = """
                    INSERT INTO `test`.`t_intra_mirror_skus` 
                    (`product_sku_id`, `product_id`, `name`, `size`, `price`, `sale_price`, `quantity`,`code`, `shop_product_sku_id`, `referer`, `date_added`) VALUES 
                    (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, now())
                """
                cursor.execute(sql, (
                    product_sku_id, product_id, name, size, price, sale_price, quantity, code, shop_product_sku_id,
                    json.dumps(sku)))
        except Exception as e:
            saveError("sku:%s" % (product_sku_id), e, sku, cursor)


def saveProducts(products, cursor):
    for index, product in enumerate(products):
        product_id = product[u'product_id']
        image_urls = json.loads(product[u'cover_img'])
        cover = image_urls[0]
        season = product[u'season']
        season_code = product[u'season_code']
        sale_at = product[u'sale_at']
        img_modified = product[u'img_modified']
        min_price = product[u'minprice']
        shop_product_id = product[u'shop_product_id']
        designer = product[u'brand_englishName']
        skus = product[u'skuList']
        name = None
        if skus:
            name = skus[0][u'name']

        # 检查商品是否已存在
        if checkProductExists(product_id, cursor):
            sql = """
              UPDATE `test`.`t_intra_mirror` SET  
              `cover`=%s, 
              `name`=%s,
              `designer`=%s, 
              `season`=%s, 
              `min_price`=%s, 
              `image_urls`=%s, 
              `shop_product_id`=%s, 
              `referer`=%s, 
              `date_modified`=now() WHERE product_id=%s
            """
            try:
                # 保存商品信息
                cursor.execute(sql, (
                    cover, name, designer, season, min_price, json.dumps(image_urls), shop_product_id,
                    json.dumps(product), product_id))
                # 保存sku 信息
                saveSkus(skus, cursor)
            except Exception as e:
                saveError('product:%s' % (product_id), e, product, cursor)
        else:
            sql = """
              INSERT INTO `test`.`t_intra_mirror` 
              (`product_id`, `cover`, `name`, `designer`, `season`, `min_price`, `image_urls`, `shop_product_id`, `referer`, `date_added`) 
              VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s,now());
            """
            try:
                # 保存商品信息
                cursor.execute(sql, (
                    product_id, cover, name, designer, season, min_price, json.dumps(image_urls), shop_product_id,
                    json.dumps(product)))
                # 保存sku 信息
                saveSkus(skus, cursor)
            except Exception as e:
                saveError('product:%s' % (product_id), e, product, cursor)

                # 下载图片
                # downloadImages(image_urls, product_id)


if __name__ == "__main__":
    spider_pages = None
    if len(sys.argv) == 2:
        spider_pages = int(sys.argv[1])

    db = pymysql.connect(host='172.16.8.147', port=3306, user='dba', passwd='123456', db='asos', charset='utf8mb4')
    # 使用cursor()方法获取操作游标
    cursor = db.cursor()
    page = urllib.urlopen("http://106.14.205.107:8083/shoplus-buyer/Home/get/getProductList?pageNumber=0&pageSize=30")
    html = page.read().decode("utf8")
    product_info = json.loads(html)
    product_list = product_info[u'productList']
    total_page = product_list[u'totalPage']
    products = product_list[u'list']
    saveProducts(products, cursor)
    db.commit()

    # 参数决定爬多少页
    total_page = spider_pages or total_page

    for i in range(1, total_page):
        url = "http://106.14.205.107:8083/shoplus-buyer/Home/get/getProductList?pageNumber=%s&pageSize=30" % (i)
        print "begin fetch url:", url
        page = urllib.urlopen(url)
        html = page.read().decode("utf8")
        product_info = json.loads(html)
        product_list = product_info[u'productList']
        products = product_list[u'list']
        saveProducts(products, cursor)
        db.commit()
        print "end fetch url:", url



# -*- coding: UTF-8 -*-
import pymysql
import json
from modules.repository import Repository


# def create_connection():
#     return pymysql.connect(host="10.26.235.6", port=3066, user='dba', passwd='123456', db='intramirror',
#                            charset='utf8mb4', cursorclass=pymysql.cursors.DictCursor)


class MonitorRepository(Repository):
    def __init__(self):
        Repository.__init__(self)

    def get_monitor_products(self):
        sql = """
        SELECT
            a.product_sn2,
            b.sn,
            a.product_id,
            b.shop_product_id
        FROM
            t_product a,
            t_shop_product b
        WHERE
            a.shop_product_id IS NOT NULL
        AND a.product_sn2 LIKE "intramirror:%"
        AND a.shop_product_id = b.shop_product_id;
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def get_platform_product(self, product_id):
        sql = """
        SELECT * FROM fashion_product.t_product WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            product = cursor.fetchone()

        if not product:
            return product

        sql = """
        SELECT * FROM t_product_sku WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            product["skus"] = cursor.fetchall()

        return product

    def set_platform_product_status(self, product_id, product_status):
        print "set product(%s) status=%s" % (product_id, product_status)
        sql = """
        UPDATE t_product SET product_status=%s,modify_time=unix_timestamp(),editor_name='crawler-for-im' WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (product_status, product_id))
            self.connection.commit()
            cursor.close()

    def set_platform_product_sizes_status(self, product_id, size, sku_status):
        sql = """
        UPDATE t_product_sku SET sku_status=%s,modify_time=unix_timestamp(),editor_name='crawler-for-im' WHERE product_id=%s AND size=%s
        """
        print "set product(%s) sku(%s) status=%s" % (product_id, size, sku_status)
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (sku_status, product_id, size))
            self.connection.commit()
            cursor.close()

    def set_platform_product_size_quantity(self, product_id, size, quantity):
        sql = """
        UPDATE t_product_sku SET quantity=%s,modify_time=unix_timestamp(),editor_name='crawler-for-im' WHERE product_id=%s AND size=%s
        """
        print "set product(%s) sku(%s) quantity=%s" % (product_id, size, quantity)
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (quantity, product_id, size))
            self.connection.commit()
            cursor.close()


class ProductRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="intramirror")

    def save_product(self, product):
        print "[%d]save product:" % product["product_id"], product["name"], product["category"]
        sql = """
        REPLACE INTO `t_product` 
        (`product_id`, `category_id`, `designer_id`, `price`, `sale_price`, `color_code`, `status`, 
        `designer_code`, `image_urls`,  `thumbnails`, `context`, `date_added`)
        VALUES 
        (%(product_id)s, %(category_id)s, %(designer_id)s, %(price)s, %(sale_price)s, %(color_code)s, %(product_status)s, %(designer_code)s, 
        %(image_urls)s, %(thumbnails)s, %(context_string)s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product)
            self.connection.commit()

    def check_description(self, description):
        sql = """
        SELECT * FROM t_product_description WHERE product_id=%(product_id)s AND language_id=%(language_id)s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, description)
            return cursor.fetchone()

    def save_description(self, description):
        # 默认设置为2
        if not description.get("language_id"):
            description["language_id"] = 2

        found_description = self.check_description(description)
        if found_description:
            description["product_description_id"] = found_description["product_description_id"]
            sql = """
            UPDATE `t_product_description` 
            SET 
            `product_id`=%(product_id)s, 
            `name`=%(product_name)s, 
            `description`=%(description)s, 
            `composition`=%(composition)s, 
            `location`=%(location)s, 
            `language_id`=%(language_id)s, 
            `edd_title`=%(edd_title)s, 
            `edd_description`=%(edd_description)s,
            `date_modified` = now()
             WHERE (`product_description_id`=%(product_description_id)s)        
            """
        else:
            sql = """
            INSERT INTO `t_product_description` 
            (`product_id`, `name`, `description`, `composition`, `location`, `language_id`, `edd_title`, 
            `edd_description`, `date_added`) 
            VALUES 
            (%(product_id)s, %(product_name)s, %(description)s, %(composition)s, %(location)s, %(language_id)s, 
            %(edd_title)s, %(edd_description)s, now());
            """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, description)
            self.connection.commit()

    def save_designer(self, designer):
        sql = """
        REPLACE INTO `t_designer` (`designer_id`, `name`,`date_added`)
         VALUES 
        (%(designer_id)s, %(designer_name)s, now() );
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, designer)
            self.connection.commit()

    def find_descriptions_zh(self):
        sql = """
        SELECT
            *
        FROM
            t_product_description
        WHERE
            language_id = 1
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def find_descriptions(self):
        sql = """
        SELECT
            *
        FROM
            t_product_description
        WHERE
            language_id = 2
        AND composition IS NOT NULL
        AND TRIM(composition) <> ""
        AND product_id NOT IN (
            SELECT
                product_id
            FROM
                t_product_description
            WHERE
                language_id = 1
        )
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()


class CategoryRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="intramirror")

    def save_categories(self, categories):
        for category in categories:
            category["name_zh"] = category["chinese_name"]
            category["category_code"] = category["show_code_int"]
            category["category_name"] = category["name"]
            category["category_level"] = category["level"]
            self.save_category(category)

    def save_category(self, category):
        print "save category:", category
        sql = """
        REPLACE INTO `t_category` 
        (`category_id`, `name`, `name_zh`, `level`, `sort_order`, `code`, `enabled`, `parent_id`, `remark`, `date_added`) 
        VALUES 
        (%(category_id)s,%(category_name)s,%(name_zh)s,%(category_level)s,%(sort_order)s,%(category_code)s,%(enabled)s,%(parent_id)s,%(remark)s,now())
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, category)
            self.connection.commit()


class CatalogRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="intramirror")
        self.catalog_creates = 0
        self.catalog_updates = 0
        self.catalog_sku_creates = 0
        self.catalog_sku_updates = 0

    def reset(self):
        self.connection.close()
        self.__init__()

    def find_catalogs_for_task(self):
        sql = """
        SELECT a.* FROM t_catalog a LEFT JOIN t_product b ON a.product_id=b.product_id 
        WHERE b.product_id IS NULL AND (a.status <> 0 OR a.status IS NULL) LIMIT 10
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def save_error(self, name, message, context):
        sql = """
        INSERT INTO `t_error` (`key`, `error`, `referer`, `date_added`, `date_modified`) VALUES 
        (%s, %s, %s, now(), now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (name, str(message), str(context)))

    def check_catalog(self, product_id):
        sql = """
        SELECT product_id FROM t_catalog WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            return cursor.fetchone()

    def check_sku(self, product_sku_id):
        sql = """
        SELECT product_sku_id FROM t_product_sku WHERE product_sku_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_sku_id)
            return cursor.fetchone()

    def save_catalog(self, catalog):
        print "save catalog", catalog["product_id"]
        # 检查商品是否已存在
        if self.check_catalog(catalog["product_id"]):
            sql = """
            UPDATE t_catalog SET  
            `cover`=%(cover)s, 
            `name`=%(product_name)s,
            `designer`=%(designer)s, 
            `season`=%(season)s, 
            `min_price`=%(min_price)s, 
            `image_urls`=%(image_urls)s, 
            `shop_product_id`=%(shop_product_id)s, 
            `context`=%(context_string)s, 
            `date_modified`=now() WHERE product_id=%(product_id)s
            """
            self.catalog_updates += 1
        else:
            sql = """
            INSERT INTO t_catalog 
            (`product_id`, `cover`, `name`, `designer`, `season`, `min_price`, `image_urls`, `shop_product_id`, `context`, 
            `date_added`) 
            VALUES (%(product_id)s, %(cover)s, %(product_name)s, %(designer)s, %(season)s, %(min_price)s, %(image_urls)s, 
            %(shop_product_id)s, %(context_string)s,now());
            """
            self.catalog_creates += 1

        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
            self.connection.commit()

    def get_catalog_by_product_id(self, product_id):
        sql = """
        SELECT * FROM t_catalog WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            return cursor.fetchone()

    def save_catalogs(self, catalogs):
        keys = ["product_id", "cover", "product_name", "designer", "season", "min_price", "image_urls",
                "shop_product_id", "context_string"]
        for product in catalogs:
            image_urls = json.loads(product[u'cover_img'])
            product["context_string"] = json.dumps(product)
            product["cover"] = image_urls[0]
            product["min_price"] = product[u'minprice']
            product["designer"] = product[u'brand_englishName']
            skus = product[u'skuList']
            product["product_name"] = skus[0][u'name'] or None
            product["image_urls"] = json.dumps(image_urls)

            #  复制到 catalog 对象
            catalog = {}
            for key in keys:
                catalog[key] = product[key]

            self.save_catalog(catalog)
            self.save_skus(skus)
            print "catalogs created:", self.catalog_creates, "updated:", self.catalog_updates, \
                "sku created:", self.catalog_sku_creates, "updated:", self.catalog_sku_updates

    def save_skus(self, skus):
        sql_for_update = """
        UPDATE t_product_sku SET  
        `product_id`=%(product_id)s, 
        `name`=%(sku_name)s, 
        `size`=%(size)s, 
        `price`=%(price)s, 
        `sale_price`=%(sale_price)s, 
        `quantity`=%(quantity)s,
        `code`=%(sku_code)s, 
        `shop_product_sku_id`=%(shop_product_sku_id)s, 
        `context`=%(context_string)s,
        `date_modified`=now() WHERE (`product_sku_id`=%(product_sku_id)s)
        """
        sql_for_insert = """
        INSERT INTO t_product_sku
        (`product_sku_id`, `product_id`, `name`, `size`, `price`, `sale_price`, `quantity`, `code`,
         `shop_product_sku_id`, `context`, `date_added`)
        VALUES
        (%(product_sku_id)s, %(product_id)s, %(sku_name)s, %(size)s, %(price)s, %(sale_price)s, %(quantity)s, 
        %(sku_code)s, %(shop_product_sku_id)s, %(context_string)s, now())
        """
        with self.connection.cursor() as cursor:
            for index, sku in enumerate(skus):
                sku["context_string"] = json.dumps(sku)
                sku["product_sku_id"] = sku[u'sku_id']
                sku["sku_name"] = sku[u'name']
                sku["quantity"] = sku[u'store'] or None
                sku["size"] = sku[u'productValue']
                if self.check_sku(sku["product_sku_id"]):
                    cursor.execute(sql_for_update, sku)
                    self.catalog_sku_updates += 1
                else:
                    cursor.execute(sql_for_insert, sku)
                    self.catalog_sku_creates += 1
            self.connection.commit()

    def save_catalog_images(self, local_path, oss_urls, product_id):
        print "save catalog images:", product_id
        sql = """
        REPLACE INTO t_product_images
        (`product_id`, `local_path`, oss_urls, `date_added`) 
        VALUES 
        (%s, %s, %s, now())
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (product_id, local_path, json.dumps(oss_urls)))
            self.connection.commit()

    def find_image_urls_for_task(self):
        sql = """
        SELECT a.product_id, a.image_urls FROM t_catalog a LEFT JOIN t_product_images b ON a.product_id = b.product_id 
        WHERE b.local_path IS NULL LIMIT 10
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def set_catalog_status(self, product_id, status):
        sql = """
        UPDATE t_catalog SET status=%s WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (status, product_id))
            self.connection.commit()


class Dao:
    def __init__(self):
        self.catalog = CatalogRepository()
        self.category = CategoryRepository()
        self.product = ProductRepository()
        self.monitor = MonitorRepository()


dao = Dao()



# -*- coding: UTF-8 -*-
import oss2

if __name__ == "__main__":
    print  oss2.__version__
    # 以下代码展示了基本的文件上传、下载、罗列、删除用法。
    # 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
    # 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
    #
    # 以杭州区域为例，Endpoint可以是：
    #   http://oss-cn-hangzhou.aliyuncs.com
    #   https://oss-cn-hangzhou.aliyuncs.com
    # 分别以HTTP、HTTPS协议访问。
    # access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
    # access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
    # bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
    # endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')

    endpoint = "http://oss-cn-beijing.aliyuncs.com	";
    access_key_id = "Ik486j07fnzdJxVC";
    access_key_secret = "zv8ESNdIrZ7wv2CZOeEMEgJhiDIoKC";
    bucket_name = "intra-mirror";
    aliasUrl = "http://cdn.fashionfinger.com";

    # 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
    bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

    # 上传一段字符串。Object名是motto.txt，内容是一段名言。
    bucket.put_object('motto.txt', 'Never give up. - Jack Ma')

    # 下载到本地文件
    bucket.get_object_to_file('motto.txt', '本地文件名.txt')
    # 上面两行代码，也可以用下面的一行代码来实现
    bucket.put_object_from_file('yunshang/云上座右铭.txt', '本地文件名.txt')


# -*- coding: UTF-8 -*-
import json

import pymysql

from keys import KEY_PRODUCT_TASK
from modules.scheduler.scheudler import RedisTaskScheduler
from modules.tools import DatetimeEncoder
from modules.repository import Repository
from modules.tools import contain_zh


#
# def create_connection():
#     return pymysql.connect(host='172.16.8.147',
#                            port=3306,
#                            user='dba',
#                            passwd='123456',
#                            db='farfetch',
#                            cursorclass=pymysql.cursors.DictCursor,
#                            charset='utf8mb4')
#
#
# class TestConnection(Repository):
#     def __init__(self, db):
#         Repository.__init__(self, host='172.16.8.147', db=db)
#         # self.connection = pymysql.connect(host='172.16.8.147',
#         #                                   port=3306,
#         #                                   user='dba',
#         #                                   passwd='123456',
#         #                                   db=db,
#         #                                   cursorclass=pymysql.cursors.DictCursor,
#         #                                   charset='utf8mb4')
#
#
# class ProductionConnection(Repository):
#     def __init__(self, db):
#         Repository.__init__(self, host='10.26.235.6', port=3066, db=db)
#         # Connection.__init__(self)
#         # self.connection = pymysql.connect(host='10.26.235.6',
#         #                                   port=3066,
#         #                                   user='dba',
#         #                                   passwd='123456',
#         #                                   db=db,
#         #                                   cursorclass=pymysql.cursors.DictCursor,
#         #                                   charset='utf8mb4')
#
#
# class RouteRepository(Repository):
#     def __init__(self):
#         Repository.__init__(self, host='10.26.235.6', port=3066, db="fashion_product")
#         # Repository.__init__(self, host='172.16.8.147', port=3306, db="fashion_product")


class MonitorRepository(Repository):
    def __init__(self):
        Repository.__init__(self)

    def set_platform_product_status(self, product_id, product_status):
        print "set product(%s) status=%s" % (product_id, product_status)
        sql = """
        UPDATE t_product SET product_status=%s,modify_time=unix_timestamp(),editor_name='monitor' WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (product_status, product_id))
            self.connection.commit()
            cursor.close()

    def get_last_crawl_time(self, product_id):
        sql = "SELECT * FROM monitor.t_product_monitor WHERE product_id=%s"
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            return cursor.fetchone()

    def update_crawl_time(self, monitor_task, status):
        sql = """
        REPLACE INTO `monitor`.`t_product_monitor` 
        (`product_id`, `source_type`, `source_id`, `status`, `date_modified`) 
        VALUES 
        (%s, %s, %s, %s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (
                monitor_task["product_id"],
                monitor_task["source_type"],
                monitor_task["resource_code"],
                status))
            self.connection.commit()

    def get_monitor_products(self):
        sql = """
        SELECT
          a.product_id,
          b.spider_product_id,
          b.source_url,
          b.resource_code,
          b.flag
        FROM
            fashion_product.t_product a,
            fashion_product.t_spider_product b
        WHERE
            a.spider_product_id IS NOT NULL
        AND a.spider_product_id = b.spider_product_id
        AND a.product_status = 1
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()


class CommonRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="farfetch")

    def get_appoint_products(self):
        sql = """SELECT * FROM spider2.t_spider_product_appoint"""
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def update_catalog_task(self, task):
        sql = """
        UPDATE farfetch.t_task SET `status` = 1,`end_time`=now() WHERE task_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, task["task_id"])
        self.connection.commit()

    def get_catalog_tasks(self):
        sql = """
        SELECT task_id,url,`gender`,start_time,name FROM farfetch.t_task WHERE `status` <> 1 OR `status` IS NULL
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()


class ProductRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="farfetch")

    def check_product(self, product):
        with self.connection.cursor() as cursor:
            sql = """
            SELECT * FROM t_product WHERE product_id=%s
            """
            cursor.execute(sql, product["product_id"])
            return cursor.fetchone()

    def create_product(self, product):
        print "create product:", product["product_id"]
        sql = """
        INSERT INTO `farfetch`.`t_product` 
        (`product_id`, 
        `cover`, 
        `status`, 
        `manufacturer_id`, 
        `category_id`, 
        `sub_category_id`, 
        `designer_style_code`, 
        `date_added`) 
        VALUES 
        (%(product_id)s,
        %(cover)s,
        %(product_status)s,
        %(manufacturer_id)s,
        %(category_id)s,
        %(sub_category_id)s,
        %(designer_style_code)s,
        now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product)
            self.connection.commit()
            cursor.close()

    def update_product(self, product):
        print "update product:", product["product_id"]
        sql = """
        UPDATE `farfetch`.`t_product` SET 
        `status`=%(product_status)s, 
        `manufacturer_id`=%(manufacturer_id)s, 
        `category_id`=%(category_id)s, 
        `sub_category_id`=%(sub_category_id)s, 
        `designer_style_code`=%(designer_style_code)s, 
        `color`=%(color)s,
        `department`=%(department)s, 
        `date_modified`=now() 
        WHERE `product_id`=%(product_id)s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product)
            self.connection.commit()
            cursor.close()

    def save_product(self, product):
        if self.check_product(product):
            self.update_product(product)
        else:
            self.create_product(product)

    def save_description(self, description):
        sql = """
        REPLACE INTO `farfetch`.`t_product_description` 
        (`product_id`, 
        `name`, 
        `description`, 
        `constitute`, 
        `location`, 
        `size_description`, 
        `language_id`, 
        `editor_name`, 
        `date_added`) 
        VALUES 
        (%(product_id)s,
        %(product_name)s,
        %(description)s,
        %(constitute)s,
        %(location)s,
        %(size_description)s,
        %(language_id)s,
        %(editor_name)s,
        now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, description)
            self.connection.commit()
            cursor.close()

    def save_manufacturer(self, manufacturer):
        sql = """
        REPLACE INTO `farfetch`.`t_manufacturer` 
        (`manufacturer_id`, `description`, `date_added`) 
        VALUES 
        (%(manufacturer_id)s, %(manufacturer)s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, manufacturer)
            self.connection.commit()
            cursor.close()

    @staticmethod
    def save_size(size, cursor):
        sql = """
        REPLACE INTO `farfetch`.`t_size` (`size_id`, `description`, `date_added`) VALUES (%(size_id)s, %(size)s, now())
        """
        cursor.execute(sql, size)

    @staticmethod
    def save_scale(scale, cursor):
        sql = """
        REPLACE INTO `farfetch`.`t_scale` (`scale_id`, `description`, `date_added`) VALUES (%(scale_id)s, %(scale)s, now())
        """
        cursor.execute(sql, scale)

    def save_product_sizes(self, sizes, product_id):
        sql = """
        REPLACE INTO `farfetch`.`t_product_sku` 
        (`product_id`, `store_id`, `size_id`, `scale_id`, `price`, `sale_price`, `quantity`, `currency`, `date_added`) 
        VALUES 
        (%(product_id)s,%(store_id)s,%(size_id)s,%(scale_id)s,%(price)s,%(sale_price)s,%(quantity)s,%(currency)s,now());
        """
        with self.connection.cursor() as cursor:
            for size in sizes:
                size["product_id"] = product_id
                cursor.execute(sql, size)
                # 保存尺码
                self.save_size(size, cursor)
                self.save_scale(size, cursor)
                self.connection.commit()
            cursor.close()

    def save_product_price(self, price):
        sql = """
        REPLACE INTO `farfetch`.`t_product_price` (`product_id`, `currency`, `price`, `sale_price`, `date_added`) 
        VALUES 
        (%(product_id)s,%(currency)s,%(price)s,%(sale_price)s,now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, price)
            self.connection.commit()
            cursor.close()

    def save_image_urls(self, image):
        sql = """
        INSERT INTO t_product_image(product_id, image_urls) VALUES(%s, %s) ON DUPLICATE KEY UPDATE date_modified=now()
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (image["product_id"], json.dumps(image["image_urls"])))
            self.connection.commit()

    def save(self, data, language_id):
        data["editor_name"] = "spider"
        data["language_id"] = language_id

        product = {
            "product_status": "CREATED",
            "designer_style_code": data.get("designerStyleId"),
            "sub_category_id": data.get("subCategoryId"),
            "manufacturer_id": data.get("manufacturerId"),
            "manufacturer": data.get("manufacturer"),
            "product_id": data["product_id"],
            "category_id": data["categoryId"],
            "department": data["department"],
            "price": data.get("unitPrice"),
            "sale_price": data.get("unitSalePrice"),
            "currency": data["currencyCode"],
            "color": data["color"],
            "cover": data["cover"],
            "image_urls": data["image_urls"]
        }

        # 如果没有尺码信息，说明没有库存了
        if not data["sizes"]:
            product["product_status"] = "OVERSOLD"

        # 保存商品信息
        self.save_product(product)

        # 保存图片信息
        self.save_image_urls(product)

        # 保存商品价格
        if product["price"] or product["sale_price"]:
            self.save_product_price(product)

        # 保存商品描述
        description = {}
        for name in ['product_id',
                     'description',
                     'constitute',
                     'location',
                     'size_description',
                     'language_id',
                     'editor_name']:
            description[name] = data[name]
        description["product_name"] = data["name"]
        self.save_description(description)

        # 保存尺码信息
        self.save_product_sizes(data["sizes"], product["product_id"])

        # 保存厂商信息
        self.save_manufacturer(product)

    def get_product_descriptions(self, language_id=1):
        sql = """
        SELECT a.resource_code,a.flag,a.source_url,b.* FROM spider2.t_spider_product a, spider2.t_spider_product_description b 
        WHERE a.spider_product_id=b.spider_product_id AND b.language_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, language_id)
            return cursor.fetchall()

    def get_product_descriptions_zh_need_translate(self):
        descriptions_zh_need_translate = []
        for description in self.get_product_descriptions():
            if not contain_zh(description["description"]):
                descriptions_zh_need_translate.append(description)
        return descriptions_zh_need_translate

    def create_appoint_products(self):
        sql = """
        REPLACE INTO farfetch.t_product_appoint (`product_id`, `status`)VALUES(%s, 0)
        """
        for product in self.get_product_descriptions_zh_need_translate():
            with self.connection.cursor() as cursor:
                if product["resource_code"]:
                    cursor.execute(sql, product["resource_code"])
        self.connection.commit()

    def get_products_from_spider(self):
        sql = """
        SELECT
          a.product_id,
          b.spider_product_id,
          b.source_url,
          b.resource_code,
          b.flag
        FROM
            fashion_product.t_product a,
            fashion_product.t_spider_product b
        WHERE
            a.spider_product_id IS NOT NULL
        AND a.spider_product_id = b.spider_product_id
        AND a.product_status = 1
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()


class ProductDescription:
    def __init__(self, connection):
        self.connection = connection

    def save(self, spider_product_id, product):
        sql = """
        REPLACE INTO `fashion_product`.`t_spider_product_description` 
        (`spider_product_id`,`name`,`description`,`constitute`,`location`,`size_description`,`language_id`,`editor_name`,`date_added`) 
        VALUES 
        (%(spider_product_id)s, %(product_name)s, %(description)s, %(constitute)s,
        %(location)s,%(size_description)s,%(language_id)s,"spider",now())
        """
        data = {
            "product_name": product['name'],
            "spider_product_id": spider_product_id
        }
        for name in ["description", "constitute", "location", "size_description", "language_id"]:
            data[name] = product[name]

        with self.connection.cursor() as cursor:
            cursor.execute(sql, data)
            self.connection.commit()


class ProductSku:
    def __init__(self, connection):
        self.connection = connection

    def save(self, spider_product_id, sku):
        sku["spider_product_id"] = spider_product_id
        sql = """
        REPLACE INTO t_spider_product_sku(`spider_product_id`,`size`,`price`,`currency`,`discount_price`,`editor_name`)
        VALUES 
        (%(spider_product_id)s,%(size)s,%(price)s,%(currency)s,%(discount_price)s,"spider")
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, sku)
            self.connection.commit()


class SpiderProductRepository(Repository):
    def __init__(self):
        Repository.__init__(db="fashion_product")
        self.description = ProductDescription(self.connection)
        self.sku = ProductSku(self.connection)
        self.created_counter = 0
        self.updated_counter = 0

    def save_product(self, product, language_id=1):
        product["language_id"] = language_id
        print json.dumps(product)
        if self.check_product(product):
            print "update product:", self.updated_counter
            self.updated_counter = self.updated_counter + 1
        else:
            # 创建商品信息
            spider_product_id = self.create_product(product)

            # 保存商品描述
            self.description.save(spider_product_id, product)
            # 只保存欧元价的sku信息
            if "EURO" == product["currency"]:
                for sku in product["sizes"]:
                    sku["currency"] = product["currency"]
                    sku["discount_price"] = product["unitSalePrice"]
                    self.sku.save(spider_product_id, sku)
            self.created_counter = self.created_counter + 1
            print "create product[%d]:" % spider_product_id, self.created_counter
            return spider_product_id

    def create_product(self, product):
        data = {}
        for name in ["designerName", "gender", "designer_code", "resource_code", "source_url"]:
            data[name] = product[name]

        # 商品是否下架
        if product["is_sold_out"]:
            data["selling_status"] = 2
        else:
            data["selling_status"] = 1

        data["category"] = product["category"] + "/" + product["subcategory"]
        data["color_code"] = product["color"]

        sql = """
        INSERT INTO fashion_product.t_spider_product
        (`designer`,`gender`,`category`,`status`,`color_code`,`designer_code`,`resource_code`, `selling_status`,`flag`,`source_url`, `editor_name`)
        VALUES
        (%(designerName)s, %(gender)s, %(category)s, 0, %(color_code)s, %(designer_code)s, %(resource_code)s,%(selling_status)s, '001', %(source_url)s, 'spider')
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, data)
            spider_product_id = cursor.lastrowid
            self.connection.commit()
            cursor.close()
            return spider_product_id

    @staticmethod
    def update_product(product):
        print product

    def check_product(self, product):
        sql = """
        SELECT * FROM t_spider_product WHERE resource_code=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product["product_id"])
            return cursor.fetchone()


class FilterRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="robin")

    def find_18ss_products(self):
        sql = """
           SELECT * FROM robin.farfetch_18ss a WHERE a.spider_product_id IS NOT NULL
           """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def find_17fw_products(self):
        sql = """
           SELECT * FROM robin.farfetch_17fw a WHERE a.spider_product_id IS NOT NULL
           """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()


class CatalogRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="farfetch")

    def create_appoint_product_tasks(self):
        sql = """
        SELECT * FROM t_product_appoint a, t_product_catalog b WHERE a.product_id = b.product_id GROUP BY a.product_id
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            for task in cursor.fetchall():
                print "create product task:", task["product_id"]
                task["url"] = "/cn" + task["url"][3:]
                task["language_id"] = 1
                RedisTaskScheduler().push(KEY_PRODUCT_TASK, json.dumps(task, cls=DatetimeEncoder))

    def create_appoint_product_tasks_from_spider2(self):
        sql = """
        SELECT
            a.product_id,
            b.source_url AS url
        FROM
            t_product_appoint a,
            spider2.t_spider_product b
        WHERE
            a.product_id = b.resource_code
        AND b.flag IN (
            SELECT
                `code`
            FROM
                t_shop_source
        )
        GROUP BY
            a.product_id
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            for task in cursor.fetchall():
                print "create product task:", task["product_id"]
                task["url"] = "/cn" + task["url"][len("https://www.farfetch.cn/cn"):]
                task["language_id"] = 1
                RedisTaskScheduler().push(KEY_PRODUCT_TASK, json.dumps(task, cls=DatetimeEncoder))

    def check_catalog(self, catalog):
        sql = "SELECT * FROM t_catalog WHERE url=%s"
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog["url"])
            return cursor.fetchone()

    def update_catalog(self, catalog):
        # print "update catalog:", catalog
        sql = """
        UPDATE `farfetch`.`t_catalog` SET 
            `url`=%(url)s, 
            `product_id`=%(product_id)s, 
            `name`=%(product_name)s, 
            `price`=%(price)s, 
            `gender`=%(gender)s, 
            `store_id`=%(store_id)s, 
            `language_id`=%(language_id)s, 
            `cover`=%(cover)s, 
            `task_id`=%(task_id)s, 
            `context`=%(catalog_context)s, 
            `date_modified`=now()
        WHERE 
            `catalog_id`=%(catalog_id)s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
        self.connection.commit()

    def create_catalog(self, catalog):
        # print "create catalog:", catalog
        sql = """
        INSERT INTO `farfetch`.`t_catalog` 
            (`url`, 
            `product_id`, 
            `name`, 
            `price`, 
            `gender`, 
            `store_id`, 
            `language_id`, 
            `cover`, 
            `task_id`, 
            `context`, 
            `date_added`) 
        VALUES 
            (%(url)s,
            %(product_id)s, 
            %(product_name)s, 
            %(price)s, 
            %(gender)s, 
            %(store_id)s, 
            %(language_id)s, 
            %(cover)s, 
            %(task_id)s, 
            %(catalog_context)s, 
            now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
        self.connection.commit()

    def save_catalog(self, catalog):
        old_catalog = self.check_catalog(catalog)
        if old_catalog:
            catalog["catalog_id"] = old_catalog["catalog_id"]
            self.update_catalog(catalog)
        else:
            self.create_catalog(catalog)

    def get_men_product_covers(self):
        sql = """
        SELECT
            a.*, b.oss_url
        FROM
            farfetch_men_cover_1000 a
        LEFT JOIN t_download_image b ON a.cover = b.image_url;
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def get_product_tasks(self):
        sql = """
        SELECT * FROM t_catalog WHERE status IS NULL LIMIT 100
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def set_catalog_status(self, catalog, status):
        sql = """
        UPDATE t_catalog SET status=%s WHERE url=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (status, catalog["url"]))
        self.connection.commit()

    def get_product_source_urls(self, product_id):
        sql = """
        SELECT * FROM farfetch.t_catalog WHERE product_id = %s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            return cursor.fetchall()


class Dao:
    def __init__(self):
        self.product = None
        self.catalog = None
        self.common = None
        self.monitor = None
        self.filter = None
        self.create_repositories()

    def create_repositories(self):
        self.filter = FilterRepository()
        self.product = ProductRepository()
        self.catalog = CatalogRepository()
        self.common = CommonRepository()
        self.monitor = MonitorRepository()

    def reset(self):
        self.create_repositories()


dao = Dao()



# -*- coding: UTF-8 -*-
import pymysql
import time
import json
import smtplib
from email.mime.text import MIMEText
from email.header import Header


class Transfer:
    def __init__(self):
        self.source_connection = pymysql.connect(host='172.16.8.147',
                                                 port=3306,
                                                 user='dba',
                                                 passwd='123456',
                                                 db='intramirror',
                                                 cursorclass=pymysql.cursors.DictCursor,
                                                 charset='utf8mb4')

        # 连接平台数据库
        self.target_connection = pymysql.connect(host='172.16.8.147',
                                                 port=3306,
                                                 user='dba',
                                                 passwd='123456',
                                                 db='fashion_product',
                                                 cursorclass=pymysql.cursors.DictCursor,
                                                 charset='utf8mb4')
        # 设计师映射表
        self.designers_map = {}
        self.categories_mapped = {}
        self.create_designers_map()
        self.create_categories_mapped()
        self.target_product_creates = 0
        self.target_product_updates = 0

    # 加载分类映射
    def create_categories_mapped(self):
        sql = "SELECT * FROM intramirror.t_category_mapped"
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql)
            for category_mapped in cursor.fetchall():
                self.categories_mapped[category_mapped["category_id"]] = category_mapped["target_category_id"]

    # 加载品牌信息
    def create_designers_map(self):
        sql = "SELECT * FROM fashion_product.t_designer"
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql)
            for designer in cursor.fetchall():
                self.designers_map[designer["name"].strip().upper()] = designer
        print "create designers map success"

    def get_source_catalogs(self):
        sql = """
        SELECT `context` FROM intramirror.t_catalog
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def create_source_season(self, catalog_context):
        catalog_context["cover_img"] = None
        catalog_context["skuList"] = None
        print "catalog context:", catalog_context
        sql = """
        REPLACE INTO t_season (`season_id`, `code`) VALUES (%(season)s, %(season_code)s)
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql, catalog_context)
        self.source_connection.commit()

    def create_source_seasons(self):
        for catalog in self.get_source_catalogs():
            self.create_source_season(json.loads(catalog["context"]))

    def get_source_designers(self):
        sql = """
        SELECT * FROM intramirror.t_designer
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def check_target_designer(self, designer):
        sql = """
        SELECT * FROM fashion_product.t_designer WHERE name=%s
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, designer["name"])
            return cursor.fetchone()

    def create_target_designer(self, designer):
        if self.check_target_designer(designer):
            print "designer exists", designer
            return

        sql = """
        INSERT INTO `fashion_product`.`t_designer` (
            `name`,`first_char`,`desc`, `create_time`,`deleted`) VALUES (%s,%s,'intra mirror',UNIX_TIMESTAMP(),'0')
        """
        print "save designer", designer
        designer_name = designer["name"]
        first_char = designer_name[1].upper()
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, (designer_name, first_char))
            self.target_connection.commit()

    # 检查商品是否存在
    def check_target_product(self, product):
        sql = """
        SELECT * FROM fashion_product.t_shop_product WHERE sn=%s LIMIT 1
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, product["sn"])
            return cursor.fetchone()

    # 获取爬虫商品sku
    def get_source_product_skus(self, product):
        sql = """
        SELECT * FROM intramirror.t_product_sku WHERE product_id=%s
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql, product["product_id"])
            return cursor.fetchall()

    # 创建商品sku
    def create_target_product_sku(self, sku, product):
        print "create target product sku:", sku
        sql = """
        INSERT INTO `fashion_product`.`t_shop_product_sku` 
        (`shop_product_id`, `size`, `price`, `settlement_price`, `sn`, `status`, `create_time`, `deleted`) 
        VALUES 
        (%(shop_product_id)s, %(size)s, %(price)s, %(sale_price)s, %(im_code)s, 'Created', unix_timestamp(), '0');
        """
        timestamp = str(int(round(time.time() * 1000)))
        sku["shop_product_id"] = product["shop_product_id"]
        sku["im_code"] = "intramirror#%s" % sku["product_sku_id"]
        sku["size"] = sku["size"]
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, sku)
            sku["shop_product_sku_id"] = cursor.lastrowid
            self.target_connection.commit()

    # 设置库存数量
    def create_product_stock(self, sku, product):
        print "create product sku stock:", sku["quantity"], ">>>>>>>>>>>>>>>"
        sql = """
           REPLACE INTO `fashion_product`.`t_shop_stock` 
           (`shop_product_id`, `shop_product_sku_id`, `shop_id`, `quantity`, `create_time`) 
           VALUES 
           (%(shop_product_id)s,%(shop_product_sku_id)s,%(shop_id)s,%(quantity)s,unix_timestamp());
           """
        sku["shop_id"] = product["shop_id"]
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, sku)
            self.target_connection.commit()

    @staticmethod
    def get_product_season_code(product):
        catalog_context = json.loads(product["catalog_context"])
        return catalog_context["season_code"] if catalog_context.get("season_code") else None

    @staticmethod
    def get_product_image_modified(product):
        catalog_context = json.loads(product["catalog_context"])
        return catalog_context["img_modified"] if catalog_context.get("img_modified") else 0

    # 加载爬虫商品列表
    def get_source_products(self):
        sql = """
        SELECT
            a.*, b.`name` AS designer_name,
            c.`name` AS product_name,
            c.description,
            c.composition,
            c.edd_description,
            c.edd_title,
            c.location,
            d.min_price,
            d.cover,
            d.context AS catalog_context
        FROM
            intramirror.t_product a,
            intramirror.t_designer b,
            intramirror.t_product_description c,
            intramirror.t_catalog d,
            intramirror.t_season e
        WHERE
            a.designer_id = b.designer_id
        AND a.product_id = c.product_id
        AND c.language_id = 1
        AND a.product_id = d.product_id
        AND d.season = e.season_id
        AND (
            (
                d.season = 173
                AND a.designer_id IN (
                    SELECT
                        designer_id
                    FROM
                        intramirror.t_designer_transfer
                )
            )
            OR d.season > 173
        )
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    # 通过品牌名称获取品牌信息
    def get_target_designer_id(self, name):
        name = name.upper()
        if self.designers_map.get(name):
            return self.designers_map[name]["designer_id"]

        sql = """
        SELECT * FROM fashion_product.t_designer WHERE `name`=%s
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, name)
            designer = cursor.fetchone()
            return designer["designer_id"] if designer else None

    def get_target_category_id(self, source_target_id):
        return self.categories_mapped[source_target_id] if self.categories_mapped.get(source_target_id) else None

    # 保存商品信息
    def save_target_product(self, product):
        # 检查商品是否存在
        product["context"] = None
        product["catalog_context"] = None
        found_target_product = self.check_target_product(product)
        if found_target_product:
            product["shop_product_id"] = found_target_product["shop_product_id"]
            self.update_target_product(product)
        else:
            self.create_target_product(product)

    def update_target_product(self, product):
        print "[%d]update product(%d)" % (product["product_id"], self.target_product_updates)
        # self.target_product_updates = self.target_product_updates + 1
        sql = """
        UPDATE `fashion_product`.`t_shop_product` SET 
        `name`=%(product_name)s, 
        `description`=%(description)s, 
        `constitute`=%(composition)s, 
        `edd_title`=%(edd_title)s, 
        `edd_description`=%(edd_description)s, 
        `location`=%(location)s, 
        `sn`=%(sn)s, 
        `designer_code`=%(designer_code)s, 
        `price`=%(min_price)s, 
        `currency`=%(currency)s, 
        `designer_id`=%(designer_id)s, 
        `category_id`=%(category_id)s, 
        `shop_id`=%(shop_id)s, 
        `status`=%(product_status)s, 
        `season`=%(season_code)s, 
        `cover`=%(cover)s, 
        `images`=%(image_urls)s, 
        `image_modified`=%(image_modified)s, 
        `modify_time`=unix_timestamp() WHERE shop_product_id=%(shop_product_id)s
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, product)
            self.target_connection.commit()
            self.target_product_updates = self.target_product_updates + 1

    def create_target_product(self, product):
        print "[%d]create product(%d)" % (product["product_id"], self.target_product_creates)
        sql = """
        INSERT INTO `fashion_product`.`t_shop_product` 
        ( `name`, `description`, `constitute`, 
        `edd_title`, `edd_description`, `location`, 
        `sn`, `designer_code`, `price`, `currency`, 
        `designer_id`, `category_id`, `shop_id`, 
        `status`, `season`, `cover`, `images`, 
        `image_modified`, `create_time`) 
        VALUES 
        (%(product_name)s, %(description)s, %(composition)s,
        %(edd_title)s, %(edd_description)s, %(location)s,
        %(sn)s, %(designer_code)s, %(min_price)s, %(currency)s,
        %(designer_id)s, %(category_id)s, %(shop_id)s,
        %(product_status)s,%(season_code)s,%(cover)s,%(image_urls)s,
        %(image_modified)s,unix_timestamp());
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, product)
            product["shop_product_id"] = cursor.lastrowid
            self.target_connection.commit()
            self.target_product_creates = self.target_product_creates + 1

        for sku in self.get_source_product_skus(product):
            self.create_target_product_sku(sku, product)
            if sku["quantity"]:
                self.create_product_stock(sku, product)

    def transfer_designers(self):
        for designer in self.get_source_designers():
            self.create_target_designer(designer)

    def transfer_products(self, shop_id):
        for product in self.get_source_products():
            designer_id = self.get_target_designer_id(product["designer_name"])
            category_id = self.get_target_category_id(product["category_id"])
            product["image_modified"] = self.get_product_image_modified(product)
            product["season_code"] = self.get_product_season_code(product)
            product["product_status"] = "Created"
            product["designer_id"] = designer_id
            product["category_id"] = category_id
            product["currency"] = "EUR"
            product["shop_id"] = shop_id
            product["sn"] = "intramirror:" + str(product["product_id"])
            # 创建商品
            self.save_target_product(product)


def send_mail():
    sender = 'yuanjie@fashionfinger.com'
    receiver = 'zhoumin@fashionfinger.com'
    subject = 'intramirror 已同步测试环境'
    smtpserver = 'smtp.fashionfinger.com'
    username = 'yuanjie@fashionfinger.com'
    password = 'qazpl,1234'

    msg = MIMEText('<html><h1>爬虫事件通知</h1></html>', 'html', 'utf-8')
    msg['Subject'] = Header(subject, 'utf-8')

    smtp = smtplib.SMTP()
    smtp.connect(smtpserver)
    smtp.login(username, password)
    smtp.sendmail(sender, receiver, msg.as_string())
    smtp.quit()


if __name__ == "__main__":
    # Transfer().transfer_designers()
    # Transfer().create_source_seasons()
    # Transfer().transfer_products(8)
    send_mail()



# -*- coding: UTF-8 -*-
import urllib
import os
import pymysql
import json
import sys
import oss2

endpoint = "http://oss-cn-beijing.aliyuncs.com	";
access_key_id = "Ik486j07fnzdJxVC";
access_key_secret = "zv8ESNdIrZ7wv2CZOeEMEgJhiDIoKC";
bucket_name = "intra-mirror";
aliasUrl = "http://cdn.fashionfinger.com";

# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)


# 内容不全重新下载
def download_with_retry(url, filename):
    try:
        urllib.urlretrieve(url, filename)
    except urllib.ContentTooShortError:
        print 'Network conditions is not good.Reloading.', url
        download_with_retry(url, filename)


def download_images(image_urls, product_id, cursor):
    dir = "spider/intra_mirror/%s" % (product_id)
    if not os.path.exists(dir):
        os.makedirs(dir)

    oss_urls = []
    for index, url in enumerate(image_urls):
        print ">>>>>>>>>>>begin download image:", url
        if url:
            names = url.split("/")
            name = names[len(names) - 1]
            path = "%s\\%s" % (dir, name)
            download_with_retry(url, path)
            print ">>>>>>>>>>>end download image:", url
            oss_url = "intra_mirror/%s/%s" % (product_id, name)
            bucket.put_object_from_file(oss_url, path)
            oss_urls.append("http://intra-mirror.oss-cn-beijing.aliyuncs.com/%s" % (oss_url))
    update_image_status(dir, oss_urls, product_id, cursor)


def update_image_status(dir, oss_urls, product_id, cursor):
    sql = """
        REPLACE INTO `test`.`t_intra_mirror_images` 
        (`product_id`,`local_dir`, oss_urls, `date_added`) 
        VALUES 
        (%s, %s, %s, now())
    """
    try:
        # 保存商品信息
        cursor.execute(sql, (product_id, dir, json.dumps(oss_urls)))
    except Exception as e:
        print e


if __name__ == "__main__":
    print sys.argv[0]
    limit_str = "limit 10"
    if len(sys.argv) == 2:
        limit_str = "limit %s" % (sys.argv[1])
    elif len(sys.argv) == 3:
        limit_str = "limit %s, %s" % (sys.argv[1], sys.argv[2])

    dbQuery = pymysql.connect(host='172.16.8.147', port=3306, user='dba', passwd='123456', db='test', charset='utf8mb4')
    cursor = dbQuery.cursor()

    dbUpdate = pymysql.connect(host='172.16.8.147', port=3306, user='dba', passwd='123456', db='test',
                               charset='utf8mb4')
    cursorUpdate = dbUpdate.cursor()
    sql = """
    select a.product_id, a.image_urls from t_intra_mirror a LEFT JOIN t_intra_mirror_images b ON a.product_id = b.product_id where b.local_dir is null %s
    """
    print "sql:", sql % (limit_str)
    try:
        cursor.execute(sql % (limit_str))
        data = cursor.fetchone()
        while data:
            image_urls = json.loads(data[1])
            download_images(image_urls, data[0], cursorUpdate)
            dbUpdate.commit()
            data = cursor.fetchone()
    except Exception as e:
        print e
    dbQuery.commit()
    dbQuery.close()
    dbUpdate.close()



from .farfetch import crawl_farfetch_product
from .intramirror import crawl_intramirror_product
from .resolver import FarfetchResolver
from mozia.crawler.repository import dao


def crawl_product(catalog):
    params = {
        "source_id" : catalog["source_id"],
        "source_type" : catalog["source_type"]
    }
    product = dao.catalog.get_product_cache(params)
    if product:
        print("use product from cache:", product)
        return product

    if 1 == catalog["source_type"]:
        product = crawl_farfetch_product(catalog)
    elif 3 == catalog["source_type"]:
        product = crawl_intramirror_product(catalog)

    if product:
        dao.catalog.save_product_cache(catalog, product)

    return product



# -*- coding: UTF-8 -*-
import json
import os
from werkzeug.utils import secure_filename
from flask import Flask, render_template, flash, redirect
from flask_bootstrap import Bootstrap
from flask_nav import Nav
from flask_nav.elements import *
import executor

app = Flask(__name__)
Bootstrap(app)

nav = Nav()

# registers the "top" menubar
nav.register_element('top', Navbar(
    View('MACROS', 'deploy')
))
nav.init_app(app)


@app.route('/deploy')
def deploy():
    return render_template('deploy.html')


@app.route('/weboff/deploy', methods=['POST'])
def deploy_weboff():
    source = json.loads(request.get_data() or "{}")
    if not source.get("source"):
        return "No source special"
    return executor.deploy_weboff(source["source"])


@app.route('/webffo/deploy', methods=['POST'])
def deploy_webffo():
    source = json.loads(request.get_data() or "{}")
    if not source.get("source"):
        return "No source special"
    return executor.deploy_webffo(source["source"])


@app.route('/webadm/deploy', methods=['POST'])
def deploy_webadm():
    source = json.loads(request.get_data() or "{}")
    if not source.get("source"):
        return "No source special"
    return executor.deploy_webadm(source["source"])


@app.route('/official/deploy', methods=['POST'])
def deploy_official():
    source = json.loads(request.get_data() or "{}")
    if not source.get("source"):
        return "No source special"
    return executor.deploy_official(source["source"])


@app.route('/finger/deploy', methods=['POST'])
def deploy_finger():
    source = json.loads(request.get_data() or "{}")
    if not source.get("source"):
        return "No source special"
    return executor.deploy_finger(source["source"])


@app.route('/webadm/restart', methods=['GET'])
def restart_webadm():
    return executor.restart_webadm()


@app.route('/weboff/restart', methods=['GET'])
def restart_weboff():
    return executor.restart_weboff()


@app.route('/webffo/restart', methods=['GET'])
def restart_webffo():
    return executor.restart_webffo()


@app.route('/webadm/logs', methods=['GET'])
def get_webadm_logs():
    return executor.get_tail_logs("/home/webadm/logs/webadm.log", 50)


@app.route('/weboff/logs', methods=['GET'])
def get_weboff_logs():
    return "No logs"


@app.route('/webffo/logs', methods=['GET'])
def get_webffo_logs():
    return executor.get_tail_logs("/home/webffo/logs/webffo.log", 50)


@app.route('/file/upload', methods=['POST'])
def uploadZip():
    fileZip = request.files['file']
    # 当前文件所在路径
    basepath = os.path.dirname(__file__)
    # 注意：没有的文件夹一定要先创建，不然会提示没有该路径
    save_path = os.path.join(basepath, 'static/zip', secure_filename(fileZip.filename))
    fileZip.save(save_path)
    return save_path


app.config['SECRET_KEY'] = 'off'
if __name__ == '__main__':
    nav.init_app(app)
    app.run(debug=False, port=8604, host="0.0.0.0")



import network
import os
from repository import dao


def download_from_oss(url, product_id):
    if not url:
        return

    file_path = "data/cover/" + os.path.basename(url)

    if not os.path.isfile(path=file_path):
        print "download image:", url
        network.download_image(url, file_path)
    else:
        print "file exists:", url


def check_product_oss_cover(product_id):
    if not product_id:
        return


def start_download_covers():
    for item in dao.catalog.get_men_product_covers():
        url = item["oss_url"]
        download_from_oss(url, item["product_id"])


if __name__ == "__main__":
    start_download_covers()



# -*- coding: UTF-8 -*-
# import json
import time
import logging
from repository import dao
# from modules.scheduler import task_scheduler
from modules.repository import repositories
from intramirror.product import crawl_product_from_monitor

KEY_INTRAMIRROR_MONITOR_TASKS = "intramirror.monitor.tasks"

monitor_task_list = []


def get_monitor_task_length():
    # return task_scheduler.length(KEY_INTRAMIRROR_MONITOR_TASKS)
    return len(monitor_task_list)


def create_monitor_tasks():
    if get_monitor_task_length() > 0:
        return

    for item in dao.monitor.get_monitor_products_from_im():
        # task_scheduler.push(KEY_INTRAMIRROR_MONITOR_TASKS, json.dumps(item))
        monitor_task_list.append(item)


def get_monitor_task():
    # task_string = task_scheduler.pop(KEY_INTRAMIRROR_MONITOR_TASKS)
    # return json.loads(task_string) if task_string else None
    return monitor_task_list.pop(0) if len(monitor_task_list) > 0 else None


def process_result(task):
    if task["event"] == "SKU_NO_FOUND" or task["event"] == "SKU_ZERO_QUANTITY_ALL":
        # 没有SKU，则下架商品
        dao.monitor.set_platform_product_status(task["product_id"], 0)
        return

    for sku in task["skus"]:
        if sku["quantity"] > 0:
            dao.monitor.set_platform_product_size_quantity(task["product_id"], sku["size"], sku["quantity"])
        else:
            dao.monitor.set_platform_product_size_status(task["product_id"], sku["size"], "OFF")


def compare_product(task, crawl_product):
    # 商品已下架， 需要处理
    if not crawl_product:
        print("product no found:", task)

    platform_product = dao.monitor.get_platform_product(task["product_id"])

    sizes_mapped = {}
    if crawl_product.get("skus"):
        for sku in crawl_product.get("skus"):
            sizes_mapped[sku["size"]] = sku

    zero_quantity_sizes = []
    platform_skus = platform_product.get("skus")
    if platform_skus:
        for sku in platform_skus:
            size_name = sku["size"]
            if not sizes_mapped.get(size_name):
                sku["quantity"] = 0
                zero_quantity_sizes.append(size_name)
                continue

            crawl_size = sizes_mapped[size_name]
            sku["quantity"] = crawl_size["count"]
            if not sku["quantity"]:
                zero_quantity_sizes.append(size_name)

    task["event"] = "UPDATE"
    # 没有SKU, 商品要下架
    if not platform_skus:
        task["event"] = "SKU_NO_FOUND"

    # 所有 SKU 库存为零
    if len(zero_quantity_sizes) == len(platform_skus):
        task["event"] = "SKU_ZERO_QUANTITY_ALL"

    # 需要列新平台商品SKU
    task["skus"] = platform_skus
    task["zero_quantity_sizes"] = zero_quantity_sizes

    return task


def exec_monitor_task(task):
    print("begin task", task)
    product_id = task["product_id"]
    crawl_product = crawl_product_from_monitor(product_id, task["url"])
    process_result(compare_product(task, crawl_product))
    print("end task", task["event"], task["product_id"])


def start_monitor_task():
    task = get_monitor_task()
    while task:
        try:
            exec_monitor_task(task)
        except Exception as e:
            print(repr(e))
        task = get_monitor_task()


def start_intramirror_monitor():
    create_monitor_tasks()
    start_monitor_task()


if __name__ == "__main__":
    repositories.connect()
    while True:
        start_intramirror_monitor()
        time.sleep(10)



# -*- coding: UTF-8 -*-
import json
import os
import time
import urllib

import oss2

from keys import KEY_DOWNLOAD_IMAGES
from modules.scheduler.scheudler import RedisTaskScheduler
from repository import create_connection

endpoint = "http://oss-cn-shanghai.aliyuncs.com";
access_key_id = "Ik486j07fnzdJxVC";
access_key_secret = "zv8ESNdIrZ7wv2CZOeEMEgJhiDIoKC";
bucket_name = "farfetch-media";
aliasUrl = "http://cdn.fashionfinger.com";

# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

connection = create_connection()


# 内容不全重新下载
def download_with_retry(url, filename):
    try:
        urllib.urlretrieve(url, filename)
    except urllib.ContentTooShortError:
        print 'Network conditions is not good.Reloading.', url
        download_with_retry(url, filename)


def download_images(image_urls, product_id):
    image_dir = "data/images/%s" % product_id
    if not os.path.exists(image_dir):
        os.makedirs(image_dir)

    oss_urls = []
    for index, image_url in enumerate(image_urls):
        print ">>>>>>>>>>>begin download image:", image_url
        if image_url:
            names = image_url.split("/")
            name = names[len(names) - 1]
            path = "%s\\%s" % (image_dir, name)
            download_with_retry(image_url, path)
            print ">>>>>>>>>>>end download image:", image_url
            oss_path = "farfetch/%s/%s" % (product_id, name)
            bucket.put_object_from_file(oss_path, path)
            oss_url = "http://farfetch-media.oss-cn-shanghai.aliyuncs.com/%s" % oss_path
            oss_urls.append(oss_url)
            save_image_url(image_url, oss_url)
            # update_image_status(dir, oss_urls, product_id, cursor)
    save_image_urls(image_urls, oss_urls, product_id)


def save_image_urls(image_urls, oss_urls, product_id):
    sql = """
    REPLACE INTO `farfetch`.`t_product_image` 
    (`product_id`, `image_urls`, `oss_urls`, `date_added`) 
    VALUES 
    (%s,%s,%s,now());
    """

    with connection.cursor() as cursor:
        cursor.execute(sql, (product_id, json.dumps(image_urls), json.dumps(oss_urls)))
        connection.commit()


def save_image_url(image_url, oss_url):
    sql = """
    REPLACE INTO `farfetch`.`t_download_image` (`image_url`, `oss_url`, `date_added`) VALUES (%s,%s,now());
    """
    with connection.cursor() as cursor:
        cursor.execute(sql, (image_url, oss_url))
        connection.commit()


def get_oss_urls(image_urls):
    length = len(image_urls)
    sql = """
    SELECT * FROM t_download_image WHERE image_url IN (%s)
    """

    with connection.cursor() as cursor:
        cursor.execute(sql % (",".join(["'" + url + "'" for url in image_urls])))
        connection.commit()
        oss_urls = cursor.fetchall()
        if length != len(oss_urls):
            return None

        return [o["oss_url"] for o in oss_urls]


if __name__ == "__main__":
    while True:
        task = RedisTaskScheduler().pop(KEY_DOWNLOAD_IMAGES)
        if not task:
            time.sleep(3)
            continue
        timestamp = int(time.time())
        task = json.loads(task)
        print "begin task:", task
        oss_urls = get_oss_urls(task["image_urls"])
        if oss_urls:
            save_image_urls(task["image_urls"], oss_urls, task["product_id"])
        else:
            download_images(task["image_urls"], task["product_id"])
        print "end task:", task, "cost:", int(time.time()) - timestamp



import json

from modules.scheduler.scheudler import RedisTaskScheduler
from modules.tools import DatetimeEncoder
from modules.tools import contain_zh
from repository import dao

task_scheduler = RedisTaskScheduler()

KEY_INTRAMIRROR_TRANSLATE_TASKS = "intramirror.translate.tasks"
KEY_INTRAMIRROR_DOWNLOAD_IMAGES_TASKS = "intramirror.download_images.tasks"


def clear_translate_tasks():
    task = task_scheduler.pop(KEY_INTRAMIRROR_TRANSLATE_TASKS)
    while task:
        task = task_scheduler.pop(KEY_INTRAMIRROR_TRANSLATE_TASKS)


def get_translate_task():
    task = task_scheduler.pop(KEY_INTRAMIRROR_TRANSLATE_TASKS)
    return json.loads(task) if task else None


def get_download_images_task():
    task = task_scheduler.pop(KEY_INTRAMIRROR_DOWNLOAD_IMAGES_TASKS)
    return json.loads(task) if task else None


def create_download_images_tasks():
    for image in dao.catalog.find_image_urls_for_task():
        print "create download images task:", image
        task_scheduler.push(KEY_INTRAMIRROR_DOWNLOAD_IMAGES_TASKS, json.dumps(image, cls=DatetimeEncoder))


def create_translate_tasks():
    for description in dao.product.find_descriptions():
        composition = description["composition"]
        if not composition:
            continue

        if contain_zh(composition):
            print "No need translate:", description["product_id"]
            continue

        task_scheduler.push(KEY_INTRAMIRROR_TRANSLATE_TASKS, json.dumps(description, cls=DatetimeEncoder))


if __name__ == '__main__':
    clear_translate_tasks()
    create_translate_tasks()



# -*- coding: UTF-8 -*-
import requests
import json
from repository import dao


def crawl_third_category():
    category_ids = [1504, 1505, 1506, 1507, 1569, 1584, 1598, 1608]
    url = "http://106.14.205.107:8083/shoplus-buyer/product/get/threeCategories?parent_category_id=%d"
    for category_id in category_ids:
        page = requests.get(url % category_id)
        category = json.loads(page.content.decode(encoding="utf-8"))
        print json.dumps(category)
        dao.category.save_categories(category["categories"])


def crawl_category():
    url = "http://106.14.205.107:8083/shoplus-buyer/product/get/categories?parent_category_id=1568"
    page = requests.get(url)
    category = json.loads(page.content.decode(encoding="utf-8"))
    print json.dumps(category)
    dao.category.save_categories(category["categories"])


if __name__ == "__main__":
    crawl_third_category()
